@@ -8,7 +8,7 @@ from anki.utils import  ids2str, splitFields, json
 from anki.hooks import runHook
 from anki import Collection
 
-class Exporter(object):
+class Exporter:
     def __init__(self, col, did=None):
         self.col = col
         self.did = did
@@ -20,10 +20,14 @@ class Exporter(object):
         file.close()
 
     def escapeText(self, text):
-        "Escape newlines, tabs and CSS."
-        text = text.replace("\n", "<br>")
+        "Escape newlines, tabs, CSS and quotechar."
+        # fixme: we should probably quote fields with newlines
+        # instead of converting them to spaces
+        text = text.replace("\n", " ")
         text = text.replace("\t", " " * 8)
         text = re.sub("(?i)<style>.*?</style>", "", text)
+        if "\"" in text:
+            text = "\"" + text.replace("\"", "\"\"") + "\""
         return text
 
     def cardIds(self):
@@ -133,7 +137,7 @@ class AnkiExporter(Exporter):
             "insert into cards values (?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)",
             data)
         # notes
-        strnids = ids2str(nids.keys())
+        strnids = ids2str(list(nids.keys()))
         notedata = []
         for row in self.src.db.all(
             "select * from notes where id in "+strnids):
@@ -199,8 +203,13 @@ class AnkiExporter(Exporter):
             if self.mediaDir:
                 for fname in os.listdir(self.mediaDir):
                     if fname.startswith("_"):
-                        media[fname] = True
-        self.mediaFiles = media.keys()
+                        # Scan all models in mids for reference to fname
+                        for m in self.src.models.all():
+                            if int(m['id']) in mids:
+                                if self._modelHasMedia(m, fname):
+                                    media[fname] = True
+                                    break
+        self.mediaFiles = list(media.keys())
         self.dst.crt = self.src.crt
         # todo: tags?
         self.count = self.dst.cardCount()
@@ -216,6 +225,16 @@ class AnkiExporter(Exporter):
     def removeSystemTags(self, tags):
         return self.src.tags.remFromStr("marked leech", tags)
 
+    def _modelHasMedia(self, model, fname):
+        # First check the styling
+        if fname in model["css"]:
+            return True
+        # If no reference to fname then check the templates as well
+        for t in model["tmpls"]:
+            if fname in t["qfmt"] or fname in t["afmt"]:
+                return True
+        return False
+
 # Packaged Anki decks
 ######################################################################
 
@@ -229,7 +248,7 @@ class AnkiPackageExporter(AnkiExporter):
 
     def exportInto(self, path):
         # open a zip file
-        z = zipfile.ZipFile(path, "w", zipfile.ZIP_DEFLATED)
+        z = zipfile.ZipFile(path, "w", zipfile.ZIP_DEFLATED, allowZip64=True)
         # if all decks and scheduling included, full export
         if self.includeSched and not self.did:
             media = self.exportVerbatim(z)
@@ -247,16 +266,13 @@ class AnkiPackageExporter(AnkiExporter):
         z.write(colfile, "collection.anki2")
         # and media
         self.prepareMedia()
-        media = {}
-        for c, file in enumerate(self.mediaFiles):
-            c = str(c)
-            mpath = os.path.join(self.mediaDir, file)
-            if os.path.exists(mpath):
-                z.write(mpath, c)
-                media[c] = file
+        media = self._exportMedia(z, self.mediaFiles, self.mediaDir)
         # tidy up intermediate files
         os.unlink(colfile)
-        os.unlink(path.replace(".apkg", ".media.db"))
+        p = path.replace(".apkg", ".media.db2")
+        if os.path.exists(p):
+            os.unlink(p)
+        os.chdir(self.mediaDir)
         shutil.rmtree(path.replace(".apkg", ".media"))
         return media
 
@@ -269,14 +285,22 @@ class AnkiPackageExporter(AnkiExporter):
         # copy all media
         if not self.includeMedia:
             return {}
-        media = {}
         mdir = self.col.media.dir()
-        for c, file in enumerate(os.listdir(mdir)):
-            c = str(c)
-            mpath = os.path.join(mdir, file)
+        return self._exportMedia(z, os.listdir(mdir), mdir)
+
+    def _exportMedia(self, z, files, fdir):
+        media = {}
+        for c, file in enumerate(files):
+            cStr = str(c)
+            mpath = os.path.join(fdir, file)
             if os.path.exists(mpath):
-                z.write(mpath, c)
-                media[c] = file
+                if re.search('\.svg$', file, re.IGNORECASE):
+                    z.write(mpath, cStr, zipfile.ZIP_DEFLATED)
+                else:
+                    z.write(mpath, cStr, zipfile.ZIP_STORED)
+                media[cStr] = file
+                runHook("exportedMediaFiles", c)
+
         return media
 
     def prepareMedia(self):
