@@ -3,15 +3,17 @@
 # License: GNU AGPL, version 3 or later; http://www.gnu.org/licenses/agpl.html
 
 import re
+import sre_constants
+
 from anki.utils import ids2str, splitFields, joinFields, intTime, fieldChecksum, stripHTMLMedia
 from anki.consts import *
 from anki.hooks import *
-import sre_constants
+
 
 # Find
 ##########################################################################
 
-class Finder(object):
+class Finder:
 
     def __init__(self, col):
         self.col = col
@@ -21,6 +23,7 @@ class Finder(object):
             deck=self._findDeck,
             mid=self._findMid,
             nid=self._findNids,
+            cid=self._findCids,
             note=self._findModel,
             prop=self._findProp,
             rated=self._findRated,
@@ -35,7 +38,7 @@ class Finder(object):
         tokens = self._tokenize(query)
         preds, args = self._where(tokens)
         if preds is None:
-            return []
+            raise Exception("invalidSearch")
         order, rev = self._order(order)
         sql = self._query(preds, order)
         try:
@@ -88,8 +91,8 @@ select distinct(n.id) from cards c, notes n where c.nid=n.id and """+preds
                         token += c
                 else:
                     inQuote = c
-            # separator
-            elif c == " ":
+            # separator (space and ideographic space)
+            elif c in (" ", '\u3000'):
                 if inQuote:
                     token += c
                 elif token:
@@ -236,18 +239,20 @@ select distinct(n.id) from cards c, notes n where c.nid=n.id and """+preds
     # Commands
     ######################################################################
 
-    def _findTag(self, (val, args)):
+    def _findTag(self, args):
+        (val, args) = args
         if val == "none":
             return 'n.tags = ""'
         val = val.replace("*", "%")
         if not val.startswith("%"):
             val = "% " + val
-        if not val.endswith("%"):
+        if not val.endswith("%") or val.endswith('\\%'):
             val += " %"
         args.append(val)
-        return "n.tags like ?"
+        return "n.tags like ? escape '\\'"
 
-    def _findCardState(self, (val, args)):
+    def _findCardState(self, args):
+        (val, args) = args
         if val in ("review", "new", "learn"):
             if val == "review":
                 n = 2
@@ -258,14 +263,17 @@ select distinct(n.id) from cards c, notes n where c.nid=n.id and """+preds
             return "type = %d" % n
         elif val == "suspended":
             return "c.queue = -1"
+        elif val == "buried":
+            return "c.queue = -2"
         elif val == "due":
             return """
 (c.queue in (2,3) and c.due <= %d) or
 (c.queue = 1 and c.due <= %d)""" % (
     self.col.sched.today, self.col.sched.dayCutoff)
 
-    def _findRated(self, (val, args)):
+    def _findRated(self, args):
         # days(:optional_ease)
+        (val, args) = args
         r = val.split(":")
         try:
             days = int(r[0])
@@ -282,7 +290,8 @@ select distinct(n.id) from cards c, notes n where c.nid=n.id and """+preds
         return ("c.id in (select cid from revlog where id>%d %s)" %
                 (cutoff, ease))
 
-    def _findAdded(self, (val, args)):
+    def _findAdded(self, args):
+        (val, args) = args
         try:
             days = int(val)
         except ValueError:
@@ -290,8 +299,9 @@ select distinct(n.id) from cards c, notes n where c.nid=n.id and """+preds
         cutoff = (self.col.sched.dayCutoff - 86400*days)*1000
         return "c.id > %d" % cutoff
 
-    def _findProp(self, (val, args)):
+    def _findProp(self, args):
         # extract
+        (val, args) = args
         m = re.match("(^.+?)(<=|>=|!=|=|<|>)(.+?$)", val)
         if not m:
             return
@@ -326,17 +336,26 @@ select distinct(n.id) from cards c, notes n where c.nid=n.id and """+preds
         args.append("%"+val+"%")
         return "(n.sfld like ? escape '\\' or n.flds like ? escape '\\')"
 
-    def _findNids(self, (val, args)):
+    def _findNids(self, args):
+        (val, args) = args
         if re.search("[^0-9,]", val):
             return
         return "n.id in (%s)" % val
 
-    def _findMid(self, (val, args)):
+    def _findCids(self, args):
+        (val, args) = args
+        if re.search("[^0-9,]", val):
+            return
+        return "c.id in (%s)" % val
+
+    def _findMid(self, args):
+        (val, args) = args
         if re.search("[^0-9]", val):
             return
         return "n.mid = %s" % val
 
-    def _findModel(self, (val, args)):
+    def _findModel(self, args):
+        (val, args) = args
         ids = []
         val = val.lower()
         for m in self.col.models.all():
@@ -344,8 +363,9 @@ select distinct(n.id) from cards c, notes n where c.nid=n.id and """+preds
                 ids.append(m['id'])
         return "n.mid in %s" % ids2str(ids)
 
-    def _findDeck(self, (val, args)):
+    def _findDeck(self, args):
         # if searching for all decks, skip
+        (val, args) = args
         if val == "*":
             return "skip"
         # deck types
@@ -365,7 +385,9 @@ select distinct(n.id) from cards c, notes n where c.nid=n.id and """+preds
         else:
             # wildcard
             ids = set()
+            # should use re.escape in the future
             val = val.replace("*", ".*")
+            val = val.replace("+", "\\+")
             for d in self.col.decks.all():
                 if re.match("(?i)"+val, d['name']):
                     ids.update(dids(d['id']))
@@ -374,8 +396,9 @@ select distinct(n.id) from cards c, notes n where c.nid=n.id and """+preds
         sids = ids2str(ids)
         return "c.did in %s or c.odid in %s" % (sids, sids)
 
-    def _findTemplate(self, (val, args)):
+    def _findTemplate(self, args):
         # were we given an ordinal number?
+        (val, args) = args
         try:
             num = int(val) - 1
         except:
@@ -415,13 +438,13 @@ select distinct(n.id) from cards c, notes n where c.nid=n.id and """+preds
         for (id,mid,flds) in self.col.db.execute("""
 select id, mid, flds from notes
 where mid in %s and flds like ? escape '\\'""" % (
-                         ids2str(mods.keys())),
+                         ids2str(list(mods.keys()))),
                          "%"+val+"%"):
             flds = splitFields(flds)
             ord = mods[str(mid)][1]
             strg = flds[ord]
             try:
-                if re.search("(?i)^"+regex+"$", strg):
+                if re.search("(?si)^"+regex+"$", strg):
                     nids.append(id)
             except sre_constants.error:
                 return
@@ -429,8 +452,9 @@ where mid in %s and flds like ? escape '\\'""" % (
             return "0"
         return "n.id in %s" % ids2str(nids)
 
-    def _findDupes(self, (val, args)):
+    def _findDupes(self, args):
         # caller must call stripHTMLMedia on passed val
+        (val, args) = args
         try:
             mid, val = val.split(",", 1)
         except OSError:
@@ -510,7 +534,7 @@ def fieldNames(col, downcase=True):
 
 # Find duplicates
 ##########################################################################
-
+# returns array of ("dupestr", [nids])
 def findDupes(col, fieldName, search=""):
     # limit search to notes with applicable field name
     if search:
@@ -536,6 +560,7 @@ def findDupes(col, fieldName, search=""):
         if ord is None:
             continue
         val = flds[ord]
+        val = stripHTMLMedia(val)
         # empty does not count as duplicate
         if not val:
             continue
