@@ -5,7 +5,6 @@
 from anki.importing.csvfile import TextImporter
 from anki.importing.apkg import AnkiPackageImporter
 from anki.importing.anki2 import Anki2Importer
-from anki.importing.anki1 import Anki1Importer
 from anki.importing.supermemo_xml import SupermemoXmlImporter
 from anki.importing.mnemo import MnemosyneImporter
 from anki.importing.pauker import PaukerImporter
@@ -13,8 +12,7 @@ from anki.lang import _
 
 Importers = (
     (_("Text separated by tabs or semicolons (*)"), TextImporter),
-    (_("Packaged Anki Deck (*.apkg *.zip)"), AnkiPackageImporter),
-    (_("Anki 1.2 Deck (*.anki)"), Anki1Importer),
+    (_("Packaged Anki Deck/Collection (*.apkg *.colpkg *.zip)"), AnkiPackageImporter),
     (_("Mnemosyne 2.0 Deck (*.db)"), MnemosyneImporter),
     (_("Supermemo XML export (*.xml)"), SupermemoXmlImporter),
     (_("Pauker 1.8 Lesson (*.pau.gz)"), PaukerImporter),
