@@ -13,7 +13,7 @@ from anki.lang import _
 class TextImporter(NoteImporter):
 
     needDelimiter = True
-    patterns = ("\t", ";")
+    patterns = ("\t", "|", ",", ";", ":")
 
     def __init__(self, *args):
         NoteImporter.__init__(self, *args)
@@ -35,13 +35,12 @@ class TextImporter(NoteImporter):
             reader = csv.reader(self.data, self.dialect, doublequote=True)
         try:
             for row in reader:
-                row = [unicode(x, "utf-8") for x in row]
                 if len(row) != self.numFields:
                     if row:
                         log.append(_(
                             "'%(row)s' had %(num1)d fields, "
                             "expected %(num2)d") % {
-                            "row": u" ".join(row),
+                            "row": " ".join(row),
                             "num1": len(row),
                             "num2": self.numFields,
                             })
@@ -49,7 +48,7 @@ class TextImporter(NoteImporter):
                     continue
                 note = self.noteFromFields(row)
                 notes.append(note)
-        except (csv.Error), e:
+        except (csv.Error) as e:
             log.append(_("Aborted: %s") % str(e))
         self.log = log
         self.ignored = ignored
@@ -68,16 +67,14 @@ class TextImporter(NoteImporter):
 
     def openFile(self):
         self.dialect = None
-        self.fileobj = open(self.file, "rbU")
+        self.fileobj = open(self.file, "r", encoding='utf-8-sig')
         self.data = self.fileobj.read()
-        if self.data.startswith(codecs.BOM_UTF8):
-            self.data = self.data[len(codecs.BOM_UTF8):]
         def sub(s):
             return re.sub("^\#.*$", "__comment", s)
         self.data = [sub(x)+"\n" for x in self.data.split("\n") if sub(x) != "__comment"]
         if self.data:
             if self.data[0].startswith("tags:"):
-                tags = unicode(self.data[0][5:], "utf8").strip()
+                tags = str(self.data[0][5:]).strip()
                 self.tagsToAdd = tags.split(" ")
                 del self.data[0]
             self.updateDelimiter()
@@ -89,14 +86,13 @@ class TextImporter(NoteImporter):
             raise Exception("unknownFormat")
         self.dialect = None
         sniffer = csv.Sniffer()
-        delims = [',', '\t', ';', ':']
         if not self.delimiter:
             try:
                 self.dialect = sniffer.sniff("\n".join(self.data[:10]),
-                                             delims)
+                                             self.patterns)
             except:
                 try:
-                    self.dialect = sniffer.sniff(self.data[0], delims)
+                    self.dialect = sniffer.sniff(self.data[0], self.patterns)
                 except:
                     pass
         if self.dialect:
@@ -117,7 +113,7 @@ class TextImporter(NoteImporter):
             reader = csv.reader(self.data, delimiter=self.delimiter, doublequote=True)
         try:
             while True:
-                row = reader.next()
+                row = next(reader)
                 if row:
                     self.numFields = len(row)
                     break
@@ -132,6 +128,6 @@ class TextImporter(NoteImporter):
 
     def noteFromFields(self, fields):
         note = ForeignNote()
-        note.fields.extend([x.strip().replace("\n", "<br>") for x in fields])
+        note.fields.extend([x for x in fields])
         note.tags.extend(self.tagsToAdd)
         return note
