@@ -3,7 +3,10 @@
 # License: GNU AGPL, version 3 or later; http://www.gnu.org/licenses/agpl.html
 
 import  cgi
-from anki.consts import NEW_CARDS_RANDOM
+
+import unicodedata
+
+from anki.consts import NEW_CARDS_RANDOM, STARTING_FACTOR
 from anki.lang import _
 from anki.utils import fieldChecksum, guid64, timestampID, \
     joinFields, intTime, splitFields
@@ -13,7 +16,7 @@ from anki.lang import ngettext
 # Stores a list of fields, tags and deck
 ######################################################################
 
-class ForeignNote(object):
+class ForeignNote:
     "An temporary object storing fields and attributes."
     def __init__(self):
         self.fields = []
@@ -21,11 +24,11 @@ class ForeignNote(object):
         self.deck = None
         self.cards = {} # map of ord -> card
 
-class ForeignCard(object):
+class ForeignCard:
     def __init__(self):
         self.due = 0
         self.ivl = 1
-        self.factor = 2500
+        self.factor = STARTING_FACTOR
         self.reps = 0
         self.lapses = 0
 
@@ -117,10 +120,17 @@ class NoteImporter(Importer):
         self._ids = []
         self._cards = []
         self._emptyNotes = False
+        dupeCount = 0
+        dupes = []
         for n in notes:
-            if not self.allowHTML:
-                for c in range(len(n.fields)):
+            for c in range(len(n.fields)):
+                if not self.allowHTML:
                     n.fields[c] = cgi.escape(n.fields[c])
+                n.fields[c] = n.fields[c].strip()
+                if not self.allowHTML:
+                    n.fields[c] = n.fields[c].replace("\n", "<br>")
+                n.fields[c] = unicodedata.normalize("NFC", n.fields[c])
+            n.tags = [unicodedata.normalize("NFC", t) for t in n.tags]
             fld0 = n.fields[fld0idx]
             csum = fieldChecksum(fld0)
             # first field must exist
@@ -151,11 +161,17 @@ class NoteImporter(Importer):
                             if data:
                                 updates.append(data)
                                 updateLog.append(updateLogTxt % fld0)
+                                dupeCount += 1
                                 found = True
-                            break
+                        elif self.importMode == 1:
+                            dupeCount += 1
                         elif self.importMode == 2:
                             # allow duplicates in this case
-                            updateLog.append(dupeLogTxt % fld0)
+                            if fld0 not in dupes:
+                                # only show message once, no matter how many
+                                # duplicates are in the collection already
+                                updateLog.append(dupeLogTxt % fld0)
+                                dupes.append(fld0)
                             found = False
             # newly add
             if not found:
@@ -183,9 +199,19 @@ class NoteImporter(Importer):
             self.col.sched.randomizeCards(did)
         else:
             self.col.sched.orderCards(did)
+
         part1 = ngettext("%d note added", "%d notes added", len(new)) % len(new)
-        part2 = ngettext("%d note updated", "%d notes updated", self.updateCount) % self.updateCount
-        self.log.append("%s, %s." % (part1, part2))
+        part2 = ngettext("%d note updated", "%d notes updated",
+                         self.updateCount) % self.updateCount
+        if self.importMode == 0:
+            unchanged = dupeCount - self.updateCount
+        elif self.importMode == 1:
+            unchanged = dupeCount
+        else:
+            unchanged = 0
+        part3 = ngettext("%d note unchanged", "%d notes unchanged",
+                         unchanged) % unchanged
+        self.log.append("%s, %s, %s." % (part1, part2, part3))
         self.log.extend(updateLog)
         if self._emptyNotes:
             self.log.append(_("""\
@@ -201,7 +227,7 @@ content in the text file to the correct fields."""))
         if not self.processFields(n):
             return
         # note id for card updates later
-        for ord, c in n.cards.items():
+        for ord, c in list(n.cards.items()):
             self._cards.append((id, ord, c))
         self.col.tags.register(n.tags)
         return [id, guid64(), self.model['id'],
@@ -213,7 +239,6 @@ content in the text file to the correct fields."""))
             "insert or replace into notes values (?,?,?,?,?,?,?,?,?,?,?)",
             rows)
 
-    # need to document that deck is ignored in this case
     def updateData(self, n, id, sflds):
         self._ids.append(id)
         if not self.processFields(n, sflds):
