@@ -1,22 +1,21 @@
 # -*- coding: utf-8 -*-
 # Copyright: Damien Elmes <anki@ichi2.net>
 # License: GNU AGPL, version 3 or later; http://www.gnu.org/licenses/agpl.html
-
+import io
 import re
-import urllib
+import traceback
+import urllib.request, urllib.parse, urllib.error
 import unicodedata
 import sys
 import zipfile
-from cStringIO import StringIO
+from io import StringIO
 
-import send2trash
 from anki.utils import checksum, isWin, isMac, json
-from anki.db import DB
+from anki.db import DB, DBError
 from anki.consts import *
 from anki.latex import mungeQA
 
-
-class MediaManager(object):
+class MediaManager:
 
     soundRegexps = ["(?i)(\[sound:(?P<fname>[^]]+)\])"]
     imgRegexps = [
@@ -34,9 +33,6 @@ class MediaManager(object):
             return
         # media directory
         self._dir = re.sub("(?i)\.(anki2)$", ".media", self.col.path)
-        # convert dir to unicode if it's not already
-        if isinstance(self._dir, str):
-            self._dir = unicode(self._dir, sys.getfilesystemencoding())
         if not os.path.exists(self._dir):
             os.makedirs(self._dir)
         try:
@@ -54,12 +50,54 @@ class MediaManager(object):
     def connect(self):
         if self.col.server:
             return
-        path = self.dir()+".db"
+        path = self.dir()+".db2"
         create = not os.path.exists(path)
         os.chdir(self._dir)
         self.db = DB(path)
         if create:
             self._initDB()
+        self.maybeUpgrade()
+
+    def _initDB(self):
+        self.db.executescript("""
+create table media (
+ fname text not null primary key,
+ csum text,           -- null indicates deleted file
+ mtime int not null,  -- zero if deleted
+ dirty int not null
+);
+
+create index idx_media_dirty on media (dirty);
+
+create table meta (dirMod int, lastUsn int); insert into meta values (0, 0);
+""")
+
+    def maybeUpgrade(self):
+        oldpath = self.dir()+".db"
+        if os.path.exists(oldpath):
+            self.db.execute('attach "../collection.media.db" as old')
+            try:
+                self.db.execute("""
+    insert into media
+     select m.fname, csum, mod, ifnull((select 1 from log l2 where l2.fname=m.fname), 0) as dirty
+     from old.media m
+     left outer join old.log l using (fname)
+     union
+     select fname, null, 0, 1 from old.log where type=1;""")
+                self.db.execute("delete from meta")
+                self.db.execute("""
+    insert into meta select dirMod, usn from old.meta
+    """)
+                self.db.commit()
+            except Exception as e:
+                # if we couldn't import the old db for some reason, just start
+                # anew
+                self.col.log("failed to import old media db:"+traceback.format_exc())
+            self.db.execute("detach old")
+            npath = "../collection.media.db.old"
+            if os.path.exists(npath):
+                os.unlink(npath)
+            os.rename("../collection.media.db", npath)
 
     def close(self):
         if self.col.server:
@@ -74,6 +112,12 @@ class MediaManager(object):
                 # may have been deleted
                 pass
 
+    def _deleteDB(self):
+        path = self.db._path
+        self.close()
+        os.unlink(path)
+        self.connect()
+
     def dir(self):
         return self._dir
 
@@ -82,24 +126,37 @@ class MediaManager(object):
             return
         import win32api, win32file
         try:
-            name = win32file.GetVolumeNameForVolumeMountPoint(self._dir[:3])
+                name = win32file.GetVolumeNameForVolumeMountPoint(self._dir[:3])
         except:
             # mapped & unmapped network drive; pray that it's not vfat
             return
         if win32api.GetVolumeInformation(name)[4].lower().startswith("fat"):
-            return True
+                return True
 
     # Adding media
     ##########################################################################
     # opath must be in unicode
 
     def addFile(self, opath):
-        return self.writeData(opath, open(opath, "rb").read())
+        with open(opath, "rb") as f:
+            return self.writeData(opath, f.read())
 
-    def writeData(self, opath, data):
+    def writeData(self, opath, data, typeHint=None):
         # if fname is a full path, use only the basename
         fname = os.path.basename(opath)
-        # make sure we write it in NFC form (on mac will autoconvert to NFD),
+
+        # if it's missing an extension and a type hint was provided, use that
+        if not os.path.splitext(fname)[1] and typeHint:
+            # mimetypes is returning '.jpe' even after calling .init(), so we'll do
+            # it manually instead
+            typeMap = {
+                "image/jpeg": ".jpg",
+                "image/png": ".png",
+            }
+            if typeHint in typeMap:
+                fname += typeMap[typeHint]
+
+        # make sure we write it in NFC form (pre-APFS Macs will autoconvert to NFD),
         # and return an NFC-encoded reference
         fname = unicodedata.normalize("NFC", fname)
         # remove any dangerous characters
@@ -115,11 +172,13 @@ class MediaManager(object):
             path = os.path.join(self.dir(), fname)
             # if it doesn't exist, copy it directly
             if not os.path.exists(path):
-                open(path, "wb").write(data)
+                with open(path, "wb") as f:
+                    f.write(data)
                 return fname
             # if it's identical, reuse
-            if checksum(open(path, "rb").read()) == csum:
-                return fname
+            with open(path, "rb") as f:
+                if checksum(f.read()) == csum:
+                    return fname
             # otherwise, increment the index in the filename
             reg = " \((\d+)\)$"
             if not re.search(reg, root):
@@ -157,15 +216,15 @@ class MediaManager(object):
         strings = []
         from anki.template.template import clozeReg
         def qrepl(m):
-            if m.group(3):
-                return "[%s]" % m.group(3)
+            if m.group(4):
+                return "[%s]" % m.group(4)
             else:
                 return "[...]"
         def arepl(m):
-            return m.group(1)
+            return m.group(2)
         for ord in ords:
             s = re.sub(clozeReg%ord, qrepl, string)
-            s = re.sub(clozeReg%".+?", "\\1", s)
+            s = re.sub(clozeReg%".+?", "\\2", s)
             strings.append(s)
         strings.append(re.sub(clozeReg%".+?", arepl, string))
         return strings
@@ -180,14 +239,17 @@ class MediaManager(object):
             txt = re.sub(reg, "", txt)
         return txt
 
-    def escapeImages(self, string):
+    def escapeImages(self, string, unescape=False):
+        if unescape:
+            fn = urllib.parse.unquote
+        else:
+            fn = urllib.parse.quote
         def repl(match):
             tag = match.group(0)
             fname = match.group("fname")
             if re.match("(https?|ftp)://", fname):
                 return tag
-            return tag.replace(
-                fname, urllib.quote(fname.encode("utf-8")))
+            return tag.replace(fname, fn(fname))
         for reg in self.imgRegexps:
             string = re.sub(reg, repl, string)
         return string
@@ -212,26 +274,32 @@ class MediaManager(object):
             allRefs.update(noteRefs)
         # loop through media folder
         unused = []
-        invalid = []
         if local is None:
             files = os.listdir(mdir)
         else:
             files = local
         renamedFiles = False
+        dirFound = False
+        warnings = []
         for file in files:
             if not local:
                 if not os.path.isfile(file):
                     # ignore directories
+                    dirFound = True
                     continue
             if file.startswith("_"):
                 # leading _ says to ignore file
                 continue
-            if not isinstance(file, unicode):
-                invalid.append(unicode(file, sys.getfilesystemencoding(), "replace"))
+
+            if self.hasIllegal(file):
+                name = file.encode(sys.getfilesystemencoding(), errors="replace")
+                name = str(name, sys.getfilesystemencoding())
+                warnings.append(
+                    _("Invalid file name, please rename: %s") % name)
                 continue
+
             nfcFile = unicodedata.normalize("NFC", file)
-            # we enforce NFC fs encoding on non-macs; on macs we'll have gotten
-            # NFD so we use the above variable for comparing references
+            # we enforce NFC fs encoding on non-macs
             if not isMac and not local:
                 if file != nfcFile:
                     # delete if we already have the NFC form, otherwise rename
@@ -252,7 +320,16 @@ class MediaManager(object):
         if renamedFiles:
             return self.check(local=local)
         nohave = [x for x in allRefs if not x.startswith("_")]
-        return (nohave, unused, invalid)
+        # make sure the media DB is valid
+        try:
+            self.findChanges()
+        except DBError:
+            self._deleteDB()
+
+        if dirFound:
+            warnings.append(
+                _("Anki does not support files in subfolders of the collection.media folder."))
+        return (nohave, unused, warnings)
 
     def _normalizeNoteRefs(self, nid):
         note = self.col.getNote(nid)
@@ -268,75 +345,6 @@ class MediaManager(object):
     def have(self, fname):
         return os.path.exists(os.path.join(self.dir(), fname))
 
-    # Media syncing - changes and removal
-    ##########################################################################
-
-    def hasChanged(self):
-        return self.db.scalar("select 1 from log limit 1")
-
-    def removed(self):
-        return self.db.list("select * from log where type = ?", MEDIA_REM)
-
-    def syncRemove(self, fnames):
-        # remove provided deletions
-        for f in fnames:
-            if os.path.exists(f):
-                send2trash.send2trash(f)
-            self.db.execute("delete from log where fname = ?", f)
-            self.db.execute("delete from media where fname = ?", f)
-        # and all locally-logged deletions, as server has acked them
-        self.db.execute("delete from log where type = ?", MEDIA_REM)
-        self.db.commit()
-
-    # Media syncing - unbundling zip files from server
-    ##########################################################################
-
-    def syncAdd(self, zipData):
-        "Extract zip data; true if finished."
-        f = StringIO(zipData)
-        z = zipfile.ZipFile(f, "r")
-        finished = False
-        meta = None
-        media = []
-        # get meta info first
-        meta = json.loads(z.read("_meta"))
-        nextUsn = int(z.read("_usn"))
-        # then loop through all files
-        for i in z.infolist():
-            if i.filename == "_meta" or i.filename == "_usn":
-                # ignore previously-retrieved meta
-                continue
-            elif i.filename == "_finished":
-                # last zip in set
-                finished = True
-            else:
-                data = z.read(i)
-                csum = checksum(data)
-                name = meta[i.filename]
-                if not isinstance(name, unicode):
-                    name = unicode(name, "utf8")
-                # normalize name for platform
-                if isMac:
-                    name = unicodedata.normalize("NFD", name)
-                else:
-                    name = unicodedata.normalize("NFC", name)
-                # save file
-                open(name, "wb").write(data)
-                # update db
-                media.append((name, csum, self._mtime(name)))
-                # remove entries from local log
-                self.db.execute("delete from log where fname = ?", name)
-        # update media db and note new starting usn
-        if media:
-            self.db.executemany(
-                "insert or replace into media values (?,?,?)", media)
-        self.setUsn(nextUsn) # commits
-        # if we have finished adding, we need to record the new folder mtime
-        # so that we don't trigger a needless scan
-        if finished:
-            self.syncMod()
-        return finished
-
     # Illegal characters
     ##########################################################################
 
@@ -346,79 +354,31 @@ class MediaManager(object):
         return re.sub(self._illegalCharReg, "", str)
 
     def hasIllegal(self, str):
-        # a file that couldn't be decoded to unicode is considered invalid
-        if not isinstance(str, unicode):
+        if re.search(self._illegalCharReg, str):
             return True
-        return not not re.search(self._illegalCharReg, str)
+        try:
+            str.encode(sys.getfilesystemencoding())
+        except UnicodeEncodeError:
+            return True
+        return False
 
-    # Media syncing - bundling zip files to send to server
-    ##########################################################################
-    # Because there's no standard filename encoding for zips, and because not
-    # all zip clients support retrieving mtime, we store the files as ascii
-    # and place a json file in the zip with the necessary information.
-
-    def zipAdded(self):
-        "Add files to a zip until over SYNC_ZIP_SIZE/COUNT. Return zip data."
-        f = StringIO()
-        z = zipfile.ZipFile(f, "w", compression=zipfile.ZIP_DEFLATED)
-        sz = 0
-        cnt = 0
-        files = {}
-        cur = self.db.execute(
-            "select fname from log where type = ?", MEDIA_ADD)
-        fnames = []
-        while 1:
-            fname = cur.fetchone()
-            if not fname:
-                # add a flag so the server knows it can clean up
-                z.writestr("_finished", "")
-                break
-            fname = fname[0]
-            # we add it as a one-element array simply to make
-            # the later forgetAdded() call easier
-            fnames.append([fname])
-            z.write(fname, str(cnt))
-            files[str(cnt)] = unicodedata.normalize("NFC", fname)
-            sz += os.path.getsize(fname)
-            if sz >= SYNC_ZIP_SIZE or cnt >= SYNC_ZIP_COUNT:
-                break
-            cnt += 1
-        z.writestr("_meta", json.dumps(files))
-        z.close()
-        return f.getvalue(), fnames
-
-    def forgetAdded(self, fnames):
-        if not fnames:
-            return
-        self.db.executemany("delete from log where fname = ?", fnames)
-        self.db.commit()
-
-    # Tracking changes (private)
+    # Tracking changes
     ##########################################################################
 
-    def _initDB(self):
-        self.db.executescript("""
-create table media (fname text primary key, csum text, mod int);
-create table meta (dirMod int, usn int); insert into meta values (0, 0);
-create table log (fname text primary key, type int);
-""")
+    def findChanges(self):
+        "Scan the media folder if it's changed, and note any changes."
+        if self._changed():
+            self._logChanges()
+
+    def haveDirty(self):
+        return self.db.scalar("select 1 from media where dirty=1 limit 1")
 
     def _mtime(self, path):
         return int(os.stat(path).st_mtime)
 
     def _checksum(self, path):
-        return checksum(open(path, "rb").read())
-
-    def usn(self):
-        return self.db.scalar("select usn from meta")
-
-    def setUsn(self, usn):
-        self.db.execute("update meta set usn = ?", usn)
-        self.db.commit()
-
-    def syncMod(self):
-        self.db.execute("update meta set dirMod = ?", self._mtime(self.dir()))
-        self.db.commit()
+        with open(path, "rb") as f:
+            return checksum(f.read())
 
     def _changed(self):
         "Return dir mtime if it has changed since the last findChanges()"
@@ -429,100 +389,182 @@ create table log (fname text primary key, type int);
             return False
         return mtime
 
-    def findChanges(self):
-        "Scan the media folder if it's changed, and note any changes."
-        if self._changed():
-            self._logChanges()
-
     def _logChanges(self):
         (added, removed) = self._changes()
-        log = []
         media = []
-        mediaRem = []
-        for f in added:
-            mt = self._mtime(f)
-            media.append((f, self._checksum(f), mt))
-            log.append((f, MEDIA_ADD))
+        for f, mtime in added:
+            media.append((f, self._checksum(f), mtime, 1))
         for f in removed:
-            mediaRem.append((f,))
-            log.append((f, MEDIA_REM))
+            media.append((f, None, 0, 1))
         # update media db
-        self.db.executemany("insert or replace into media values (?,?,?)",
+        self.db.executemany("insert or replace into media values (?,?,?,?)",
                             media)
-        if mediaRem:
-            self.db.executemany("delete from media where fname = ?",
-                                mediaRem)
         self.db.execute("update meta set dirMod = ?", self._mtime(self.dir()))
-        # and logs
-        self.db.executemany("insert or replace into log values (?,?)", log)
         self.db.commit()
 
     def _changes(self):
         self.cache = {}
         for (name, csum, mod) in self.db.execute(
-            "select * from media"):
-            self.cache[name] = [csum, mod, False]
+            "select fname, csum, mtime from media where csum is not null"):
+            # previous entries may not have been in NFC form
+            normname = unicodedata.normalize("NFC", name)
+            self.cache[normname] = [csum, mod, False]
         added = []
         removed = []
         # loop through on-disk files
-        for f in os.listdir(self.dir()):
-            # ignore folders and thumbs.db
-            if os.path.isdir(f):
-                continue
-            if f.lower() == "thumbs.db":
-                continue
-            # and files with invalid chars
-            if self.hasIllegal(f):
-                continue
-            # empty files are invalid; clean them up and continue
-            if not os.path.getsize(f):
-                os.unlink(f)
-                continue
-            # newly added?
-            if f not in self.cache:
-                added.append(f)
-            else:
-                # modified since last time?
-                if self._mtime(f) != self.cache[f][1]:
-                    # and has different checksum?
-                    if self._checksum(f) != self.cache[f][0]:
-                        added.append(f)
-                # mark as used
-                self.cache[f][2] = True
+        with os.scandir(self.dir()) as it:
+            for f in it:
+                # ignore folders and thumbs.db
+                if f.is_dir():
+                    continue
+                if f.name.lower() == "thumbs.db":
+                    continue
+                # and files with invalid chars
+                if self.hasIllegal(f.name):
+                    continue
+                # empty files are invalid; clean them up and continue
+                sz = f.stat().st_size
+                if not sz:
+                    os.unlink(f.name)
+                    continue
+                if sz > 100*1024*1024:
+                    self.col.log("ignoring file over 100MB", f.name)
+                    continue
+                # check encoding
+                normname = unicodedata.normalize("NFC", f.name)
+                if not isMac:
+                    if f.name != normname:
+                        # wrong filename encoding which will cause sync errors
+                        if os.path.exists(normname):
+                            os.unlink(f.name)
+                        else:
+                            os.rename(f.name, normname)
+                else:
+                    # on Macs we can access the file using any normalization
+                    pass
+
+                # newly added?
+                mtime = int(f.stat().st_mtime)
+                if normname not in self.cache:
+                    added.append((normname, mtime))
+                else:
+                    # modified since last time?
+                    if mtime != self.cache[normname][1]:
+                        # and has different checksum?
+                        if self._checksum(normname) != self.cache[normname][0]:
+                            added.append((normname, mtime))
+                    # mark as used
+                    self.cache[normname][2] = True
         # look for any entries in the cache that no longer exist on disk
-        for (k, v) in self.cache.items():
+        for (k, v) in list(self.cache.items()):
             if not v[2]:
                 removed.append(k)
         return added, removed
 
-    def sanityCheck(self):
-        assert not self.db.scalar("select count() from log")
-        cnt = self.db.scalar("select count() from media")
-        return cnt
+    # Syncing-related
+    ##########################################################################
+
+    def lastUsn(self):
+        return self.db.scalar("select lastUsn from meta")
+
+    def setLastUsn(self, usn):
+        self.db.execute("update meta set lastUsn = ?", usn)
+        self.db.commit()
+
+    def syncInfo(self, fname):
+        ret = self.db.first(
+            "select csum, dirty from media where fname=?", fname)
+        return ret or (None, 0)
+
+    def markClean(self, fnames):
+        for fname in fnames:
+            self.db.execute(
+                "update media set dirty=0 where fname=?", fname)
+
+    def syncDelete(self, fname):
+        if os.path.exists(fname):
+            os.unlink(fname)
+        self.db.execute("delete from media where fname=?", fname)
+
+    def mediaCount(self):
+        return self.db.scalar(
+            "select count() from media where csum is not null")
+
+    def dirtyCount(self):
+        return self.db.scalar(
+            "select count() from media where dirty=1")
 
     def forceResync(self):
         self.db.execute("delete from media")
-        self.db.execute("delete from log")
-        self.db.execute("update meta set usn = 0, dirMod = 0")
+        self.db.execute("update meta set lastUsn=0,dirMod=0")
         self.db.commit()
+        self.db.setAutocommit(True)
+        self.db.execute("vacuum")
+        self.db.execute("analyze")
+        self.db.setAutocommit(False)
 
-    def removeExisting(self, files):
-        "Remove files from list of files to sync, and return missing files."
-        need = []
-        remove = []
-        for f in files:
-            if isMac:
-                name = unicodedata.normalize("NFD", f)
+    # Media syncing: zips
+    ##########################################################################
+
+    def mediaChangesZip(self):
+        f = io.BytesIO()
+        z = zipfile.ZipFile(f, "w", compression=zipfile.ZIP_DEFLATED)
+
+        fnames = []
+        # meta is list of (fname, zipname), where zipname of None
+        # is a deleted file
+        meta = []
+        sz = 0
+
+        for c, (fname, csum) in enumerate(self.db.execute(
+                        "select fname, csum from media where dirty=1"
+                        " limit %d"%SYNC_ZIP_COUNT)):
+
+            fnames.append(fname)
+            normname = unicodedata.normalize("NFC", fname)
+
+            if csum:
+                self.col.log("+media zip", fname)
+                z.write(fname, str(c))
+                meta.append((normname, str(c)))
+                sz += os.path.getsize(fname)
             else:
-                name = f
-            if self.db.scalar("select 1 from log where fname=?", name):
-                remove.append((name,))
+                self.col.log("-media zip", fname)
+                meta.append((normname, ""))
+
+            if sz >= SYNC_ZIP_SIZE:
+                break
+
+        z.writestr("_meta", json.dumps(meta))
+        z.close()
+        return f.getvalue(), fnames
+
+    def addFilesFromZip(self, zipData):
+        "Extract zip data; true if finished."
+        f = io.BytesIO(zipData)
+        z = zipfile.ZipFile(f, "r")
+        media = []
+        # get meta info first
+        meta = json.loads(z.read("_meta").decode("utf8"))
+        # then loop through all files
+        cnt = 0
+        for i in z.infolist():
+            if i.filename == "_meta":
+                # ignore previously-retrieved meta
+                continue
             else:
-                need.append(f)
-        self.db.executemany("delete from log where fname=?", remove)
-        self.db.commit()
-        # if we need all the server files, it's faster to pass None than
-        # the full list
-        if need and len(files) == len(need):
-            return None
-        return need
+                data = z.read(i)
+                csum = checksum(data)
+                name = meta[i.filename]
+                # normalize name
+                name = unicodedata.normalize("NFC", name)
+                # save file
+                with open(name, "wb") as f:
+                    f.write(data)
+                # update db
+                media.append((name, csum, self._mtime(name), 0))
+                cnt += 1
+        if media:
+            self.db.executemany(
+                "insert or replace into media values (?,?,?,?)", media)
+        return cnt
