@@ -69,7 +69,7 @@ defaultTemplate = {
     #'bsize': 12,
 }
 
-class ModelManager(object):
+class ModelManager:
 
     # Saving/loading registry
     #############################################################
@@ -108,7 +108,7 @@ class ModelManager(object):
         m = self.get(self.col.decks.current().get('mid'))
         if not forDeck or not m:
             m = self.get(self.col.conf['curModel'])
-        return m or self.models.values()[0]
+        return m or list(self.models.values())[0]
 
     def setCurrent(self, m):
         self.col.conf['curModel'] = m['id']
@@ -122,14 +122,14 @@ class ModelManager(object):
 
     def all(self):
         "Get all models."
-        return self.models.values()
+        return list(self.models.values())
 
     def allNames(self):
         return [m['name'] for m in self.all()]
 
     def byName(self, name):
         "Get model with NAME."
-        for m in self.models.values():
+        for m in list(self.models.values()):
             if m['name'] == name:
                 return m
 
@@ -147,7 +147,7 @@ class ModelManager(object):
 
     def rem(self, m):
         "Delete model, and all its cards/notes."
-        self.col.modSchema()
+        self.col.modSchema(check=True)
         current = self.current()['id'] == m['id']
         # delete notes/cards
         self.col.remCards(self.col.db.list("""
@@ -158,7 +158,7 @@ select id from cards where nid in (select id from notes where mid = ?)""",
         self.save()
         # GUI should ensure last model is not deleted
         if current:
-            self.setCurrent(self.models.values()[0])
+            self.setCurrent(list(self.models.values())[0])
 
     def add(self, m):
         self._setID(m)
@@ -191,7 +191,7 @@ select id from cards where nid in (select id from notes where mid = ?)""",
         return str(id) in self.models
 
     def ids(self):
-        return self.models.keys()
+        return list(self.models.keys())
 
     # Tools
     ##################################################
@@ -241,7 +241,7 @@ and notes.mid = ? and cards.ord = ?""", m['id'], ord)
 
     def setSortIdx(self, m, idx):
         assert idx >= 0 and idx < len(m['flds'])
-        self.col.modSchema()
+        self.col.modSchema(check=True)
         m['sortf'] = idx
         self.col.updateFieldCache(self.nids(m))
         self.save(m)
@@ -249,7 +249,7 @@ and notes.mid = ? and cards.ord = ?""", m['id'], ord)
     def addField(self, m, field):
         # only mod schema if model isn't new
         if m['id']:
-            self.col.modSchema()
+            self.col.modSchema(check=True)
         m['flds'].append(field)
         self._updateFieldOrds(m)
         self.save(m)
@@ -259,7 +259,7 @@ and notes.mid = ? and cards.ord = ?""", m['id'], ord)
         self._transformFields(m, add)
 
     def remField(self, m, field):
-        self.col.modSchema()
+        self.col.modSchema(check=True)
         # save old sort field
         sortFldName = m['flds'][m['sortf']]['name']
         idx = m['flds'].index(field)
@@ -282,7 +282,7 @@ and notes.mid = ? and cards.ord = ?""", m['id'], ord)
         self.renameField(m, field, None)
 
     def moveField(self, m, field, idx):
-        self.col.modSchema()
+        self.col.modSchema(check=True)
         oldidx = m['flds'].index(field)
         if oldidx == idx:
             return
@@ -303,11 +303,11 @@ and notes.mid = ? and cards.ord = ?""", m['id'], ord)
         self._transformFields(m, move)
 
     def renameField(self, m, field, newName):
-        self.col.modSchema()
-        pat = r'{{([:#^/]|[^:#/^}][^:}]*?:|)%s}}'
+        self.col.modSchema(check=True)
+        pat = r'{{([^{}]*)([:#^/]|[^:#/^}][^:}]*?:|)%s}}'
         def wrap(txt):
             def repl(match):
-                return '{{' + match.group(1) + txt +  '}}'
+                return '{{' + match.group(1) + match.group(2) + txt +  '}}'
             return repl
         for t in m['tmpls']:
             for fmt in ('qfmt', 'afmt'):
@@ -347,7 +347,7 @@ and notes.mid = ? and cards.ord = ?""", m['id'], ord)
     def addTemplate(self, m, template):
         "Note: should col.genCards() afterwards."
         if m['id']:
-            self.col.modSchema()
+            self.col.modSchema(check=True)
         m['tmpls'].append(template)
         self._updateTemplOrds(m)
         self.save(m)
@@ -370,7 +370,7 @@ having count() < 2
 limit 1""" % ids2str(cids)):
             return False
         # ok to proceed; remove cards
-        self.col.modSchema()
+        self.col.modSchema(check=True)
         self.col.remCards(cids)
         # shift ordinals
         self.col.db.execute("""
@@ -414,7 +414,7 @@ select id from notes where mid = ?)""" % " ".join(map),
     # - newModel should be self if model is not changing
 
     def change(self, m, nids, newModel, fmap, cmap):
-        self.col.modSchema()
+        self.col.modSchema(check=True)
         assert newModel['id'] == m['id'] or (fmap and cmap)
         if fmap:
             self._changeNotes(nids, newModel, fmap)
@@ -429,7 +429,7 @@ select id from notes where mid = ?)""" % " ".join(map),
             "select id, flds from notes where id in "+ids2str(nids)):
             newflds = {}
             flds = splitFields(flds)
-            for old, new in map.items():
+            for old, new in list(map.items()):
                 newflds[new] = flds[old]
             flds = []
             for c in range(nfields):
