@@ -2,7 +2,6 @@
 # Copyright: Damien Elmes <anki@ichi2.net>
 # License: GNU AGPL, version 3 or later; http://www.gnu.org/licenses/agpl.html
 
-from __future__ import division
 import time
 import datetime
 import json
@@ -15,7 +14,7 @@ from anki.lang import _, ngettext
 # Card stats
 ##########################################################################
 
-class CardStats(object):
+class CardStats:
 
     def __init__(self, col, card):
         self.col = col
@@ -99,7 +98,7 @@ colTime = "#770"
 colUnseen = "#000"
 colSusp = "#ff0"
 
-class CollectionStats(object):
+class CollectionStats:
 
     def __init__(self, col):
         self.col = col
@@ -112,24 +111,31 @@ class CollectionStats(object):
     def report(self, type=0):
         # 0=days, 1=weeks, 2=months
         self.type = type
-        from statsbg import bg
+        from .statsbg import bg
         txt = self.css % bg
-        txt += self.todayStats()
-        txt += self.dueGraph()
-        txt += self.repsGraph()
-        txt += self.ivlGraph()
-        txt += self.hourGraph()
-        txt += self.easeGraph()
-        txt += self.cardGraph()
-        txt += self.footer()
+        txt += self._section(self.todayStats())
+        txt += self._section(self.dueGraph())
+        txt += self.repsGraphs()
+        txt += self._section(self.introductionGraph())
+        txt += self._section(self.ivlGraph())
+        txt += self._section(self.hourGraph())
+        txt += self._section(self.easeGraph())
+        txt += self._section(self.cardGraph())
+        txt += self._section(self.footer())
         return "<script>%s\n</script><center>%s</center>" % (
             anki.js.jquery+anki.js.plot, txt)
 
+    def _section(self, txt):
+        return "<div class=section>%s</div>" % txt
+
     css = """
 <style>
 h1 { margin-bottom: 0; margin-top: 1em; }
 .pielabel { text-align:center; padding:0px; color:white; }
 body {background-image: url(data:image/png;base64,%s); }
+@media print {
+    .section { page-break-inside: avoid; padding-top: 5mm; }
+}
 </style>
 """
 
@@ -159,7 +165,7 @@ from revlog where id > ? """+lim, (self.col.sched.dayCutoff-86400)*1000)
         filt = filt or 0
         # studied
         def bold(s):
-            return "<b>"+unicode(s)+"</b>"
+            return "<b>"+str(s)+"</b>"
         msgp1 = ngettext("<!--studied-->%d card", "<!--studied-->%d cards", cards) % cards
         b += _("Studied %(a)s in %(b)s today.") % dict(
             a=bold(msgp1), b=bold(fmtTimeSpan(thetime, unit=1)))
@@ -254,22 +260,62 @@ group by day order by day""" % (self._limit(), lim),
                             today=self.col.sched.today,
                             chunk=chunk)
 
-    # Reps and time spent
+    # Added, reps and time spent
     ######################################################################
 
-    def repsGraph(self):
+    def introductionGraph(self):
         if self.type == 0:
             days = 30; chunk = 1
         elif self.type == 1:
             days = 52; chunk = 7
         else:
             days = None; chunk = 30
-        return self._repsGraph(self._done(days, chunk),
+        return self._introductionGraph(self._added(days, chunk),
+                               days, _("Added"))
+
+    def _introductionGraph(self, data, days, title):
+        if not data:
+            return ""
+        d = data
+        conf = dict(
+            xaxis=dict(tickDecimals=0, max=0.5),
+            yaxes=[dict(min=0), dict(position="right",min=0)])
+        if days is not None:
+            conf['xaxis']['min'] = -days+0.5
+        def plot(id, data, ylabel, ylabel2):
+            return self._graph(
+                id, data=data, conf=conf, ylabel=ylabel, ylabel2=ylabel2)
+        # graph
+        (repdata, repsum) = self._splitRepData(d, ((1, colLearn, ""),))
+        txt = self._title(
+            title, _("The number of new cards you have added."))
+        txt += plot("intro", repdata, ylabel=_("Cards"), ylabel2=_("Cumulative Cards"))
+        # total and per day average
+        tot = sum([i[1] for i in d])
+        period = self._periodDays()
+        if not period:
+            # base off date of earliest added card
+            period = self._deckAge('add')
+        i = []
+        self._line(i, _("Total"), ngettext("%d card", "%d cards", tot) % tot)
+        self._line(i, _("Average"), self._avgDay(tot, period, _("cards")))
+        txt += self._lineTbl(i)
+
+        return txt
+
+    def repsGraphs(self):
+        if self.type == 0:
+            days = 30; chunk = 1
+        elif self.type == 1:
+            days = 52; chunk = 7
+        else:
+            days = None; chunk = 30
+        return self._repsGraphs(self._done(days, chunk),
                                days,
                                _("Review Count"),
                                _("Review Time"))
 
-    def _repsGraph(self, data, days, reptitle, timetitle):
+    def _repsGraphs(self, data, days, reptitle, timetitle):
         if not data:
             return ""
         d = data
@@ -288,13 +334,13 @@ group by day order by day""" % (self._limit(), lim),
             (4, colRelearn, _("Relearn")),
             (1, colLearn, _("Learn")),
             (5, colCram, _("Cram"))))
-        txt = self._title(
+        txt1 = self._title(
             reptitle, _("The number of questions you have answered."))
-        txt += plot("reps", repdata, ylabel=_("Answers"), ylabel2=_(
+        txt1 += plot("reps", repdata, ylabel=_("Answers"), ylabel2=_(
             "Cumulative Answers"))
         (daysStud, fstDay) = self._daysStudied()
         rep, tot = self._ansInfo(repsum, daysStud, fstDay, _("reviews"))
-        txt += rep
+        txt1 += rep
         # time
         (timdata, timsum) = self._splitRepData(d, (
             (8, colMature, _("Mature")),
@@ -308,12 +354,12 @@ group by day order by day""" % (self._limit(), lim),
         else:
             t = _("Hours")
             convHours = True
-        txt += self._title(timetitle, _("The time taken to answer the questions."))
-        txt += plot("time", timdata, ylabel=t, ylabel2=_("Cumulative %s") % t)
+        txt2 = self._title(timetitle, _("The time taken to answer the questions."))
+        txt2 += plot("time", timdata, ylabel=t, ylabel2=_("Cumulative %s") % t)
         rep, tot2 = self._ansInfo(
             timsum, daysStud, fstDay, _("minutes"), convHours, total=tot)
-        txt += rep
-        return txt
+        txt2 += rep
+        return self._section(txt1) + self._section(txt2)
 
     def _ansInfo(self, totd, studied, first, unit, convHours=False, total=None):
         if not totd:
@@ -322,15 +368,7 @@ group by day order by day""" % (self._limit(), lim),
         period = self._periodDays()
         if not period:
             # base off earliest repetition date
-            lim = self._revlogLimit()
-            if lim:
-                lim = " where " + lim
-            t = self.col.db.scalar("select id from revlog %s order by id limit 1" % lim)
-            if not t:
-                period = 1
-            else:
-                period = max(
-                    1, int(1+((self.col.sched.dayCutoff - (t/1000)) / 86400)))
+            period = self._deckAge('review')
         i = []
         self._line(i, _("Days studied"),
                    _("<b>%(pct)d%%</b> (%(x)s of %(y)s)") % dict(
@@ -353,10 +391,15 @@ group by day order by day""" % (self._limit(), lim),
                 tot, period, unit))
         if total and tot:
             perMin = total / float(tot)
-            perMin = ngettext("%d card/minute", "%d cards/minute", perMin) % perMin
+            perMin = round(perMin, 1)
+            # don't round down to zero
+            if perMin < 0.1:
+                text = _("less than 0.1 cards/minute")
+            else:
+                text = _("%.01f cards/minute") % perMin
             self._line(
                 i, _("Average answer time"),
-                _("%(a)0.1fs (%(b)s)") % dict(a=(tot*60)/total, b=perMin))
+                _("%(a)0.1fs (%(b)s)") % dict(a=(tot*60)/total, b=text))
         return self._lineTbl(i), int(tot)
 
     def _splitRepData(self, data, spec):
@@ -389,6 +432,27 @@ group by day order by day""" % (self._limit(), lim),
                 bars={'show': False}, lines=dict(show=True), stack=-n))
         return (ret, alltot)
 
+    def _added(self, num=7, chunk=1):
+        lims = []
+        if num is not None:
+            lims.append("id > %d" % (
+                (self.col.sched.dayCutoff-(num*chunk*86400))*1000))
+        lims.append("did in %s" % self._limit())
+        if lims:
+            lim = "where " + " and ".join(lims)
+        else:
+            lim = ""
+        if self.type == 0:
+            tf = 60.0 # minutes
+        else:
+            tf = 3600.0 # hours
+        return self.col.db.all("""
+select
+(cast((id/1000.0 - :cut) / 86400.0 as int))/:chunk as day,
+count(id)
+from cards %s
+group by day order by day""" % lim, cut=self.col.sched.dayCutoff,tf=tf, chunk=chunk)
+
     def _done(self, num=7, chunk=1):
         lims = []
         if num is not None:
@@ -655,7 +719,7 @@ group by hour having count() > 30 order by hour""" % lim,
             (_("Mature"), colMature),
             (_("Young+Learn"), colYoung),
             (_("Unseen"), colUnseen),
-            (_("Suspended"), colSusp))):
+            (_("Suspended+Buried"), colSusp))):
             d.append(dict(data=div[c], label="%s: %s" % (t, div[c]), color=col))
         # text data
         i = []
@@ -805,8 +869,12 @@ $(function () {
         }
     }
     conf.yaxis.minTickSize = 1;
+    // prevent ticks from having decimals (use whole numbers instead)
+    conf.yaxis.tickDecimals = 0;
     conf.yaxis.tickFormatter = function (val, axis) {
-            return val.toFixed(0);
+            // Just in case we get ticks with decimals, render to one decimal position.  If it's
+            // a whole number then render without any decimal (i.e. without the trailing .0).
+            return val === Math.round(val) ? val.toFixed(0) : val.toFixed(1);
     }
     if (conf.series.pie) {
         conf.series.pie.label.formatter = function(label, series){
@@ -834,6 +902,22 @@ $(function () {
     def _title(self, title, subtitle=""):
         return '<h1>%s</h1>%s' % (title, subtitle)
 
+    def _deckAge(self, by):
+        lim = self._revlogLimit()
+        if lim:
+            lim = " where " + lim
+        if by == 'review':
+            t = self.col.db.scalar("select id from revlog %s order by id limit 1" % lim)
+        elif by == 'add':
+            lim = "where did in %s" % ids2str(self.col.decks.active())
+            t = self.col.db.scalar("select id from cards %s order by id limit 1" % lim)
+        if not t:
+            period = 1
+        else:
+            period = max(
+                1, int(1+((self.col.sched.dayCutoff - (t/1000)) / 86400)))
+        return period
+
     def _periodDays(self):
         if self.type == 0:
             return 30
