@@ -2,97 +2,27 @@
 # Copyright: Damien Elmes <anki@ichi2.net>
 # License: GNU AGPL, version 3 or later; http://www.gnu.org/licenses/agpl.html
 
-import urllib
-import sys
+import io
 import gzip
 import random
-from cStringIO import StringIO
+import requests
 
-import httplib2
-from anki.db import DB
-from anki.utils import ids2str, intTime, json, isWin, isMac, platDesc, checksum
+from anki.db import DB, DBError
+from anki.utils import ids2str, intTime, json, platDesc, checksum, devMode
 from anki.consts import *
-from hooks import runHook
+from .hooks import runHook
 import anki
-
+from .lang import ngettext
 
 # syncing vars
 HTTP_TIMEOUT = 90
 HTTP_PROXY = None
-
-# badly named; means no retries
-httplib2.RETRIES = 1
-
-try:
-    # httplib2 >=0.7.7
-    _proxy_info_from_environment = httplib2.proxy_info_from_environment
-    _proxy_info_from_url = httplib2.proxy_info_from_url
-except AttributeError:
-    # httplib2 <0.7.7
-    _proxy_info_from_environment = httplib2.ProxyInfo.from_environment
-    _proxy_info_from_url = httplib2.ProxyInfo.from_url
-
-# Httplib2 connection object
-######################################################################
-
-def httpCon():
-    certs = os.path.join(os.path.dirname(__file__), "ankiweb.certs")
-    if not os.path.exists(certs):
-        if isWin:
-            certs = os.path.join(
-                os.path.dirname(os.path.abspath(sys.argv[0])),
-                "ankiweb.certs")
-        elif isMac:
-            certs = os.path.join(
-                os.path.dirname(os.path.abspath(sys.argv[0])),
-                "../Resources/ankiweb.certs")
-        else:
-            assert 0, "Your distro has not packaged Anki correctly."
-    return httplib2.Http(
-        timeout=HTTP_TIMEOUT, ca_certs=certs,
-        proxy_info=HTTP_PROXY,
-        disable_ssl_certificate_validation=not not HTTP_PROXY)
-
-# Proxy handling
-######################################################################
-
-def _setupProxy():
-    global HTTP_PROXY
-    # set in env?
-    p = _proxy_info_from_environment()
-    if not p:
-        # platform-specific fetch
-        url = None
-        if isWin:
-            r = urllib.getproxies_registry()
-            if 'https' in r:
-                url = r['https']
-            elif 'http' in r:
-                url = r['http']
-        elif isMac:
-            r = urllib.getproxies_macosx_sysconf()
-            if 'https' in r:
-                url = r['https']
-            elif 'http' in r:
-                url = r['http']
-        if url:
-            p = _proxy_info_from_url(url, _proxyMethod(url))
-    if p:
-        p.proxy_rdns = True
-    HTTP_PROXY = p
-
-def _proxyMethod(url):
-    if url.lower().startswith("https"):
-        return "https"
-    else:
-        return "http"
-
-_setupProxy()
+HTTP_BUF_SIZE = 64*1024
 
 # Incremental syncing
 ##########################################################################
 
-class Syncer(object):
+class Syncer:
 
     def __init__(self, col, server=None):
         self.col = col
@@ -105,29 +35,27 @@ class Syncer(object):
         # if the deck has any pending changes, flush them first and bump mod
         # time
         self.col.save()
+
         # step 1: login & metadata
         runHook("sync", "login")
         meta = self.server.meta()
         self.col.log("rmeta", meta)
         if not meta:
             return "badAuth"
+        # server requested abort?
+        self.syncMsg = meta['msg']
+        if not meta['cont']:
+            return "serverAbort"
+        else:
+            # don't abort, but if 'msg' is not blank, gui should show 'msg'
+            # after sync finishes and wait for confirmation before hiding
+            pass
         rscm = meta['scm']
         rts = meta['ts']
         self.rmod = meta['mod']
         self.maxUsn = meta['usn']
-        self.mediaUsn = meta['musn']
-        self.syncMsg = meta['msg']
-        # this is a temporary measure to address the problem of users
-        # forgetting which email address they've used - it will be removed
-        # when enough time has passed
         self.uname = meta.get("uname", "")
-        # server requested abort?
-        if not meta['cont']:
-            return "serverAbort"
-        else:
-            # don't abort, but ui should show message after sync finishes
-            # and require confirmation if it's non-empty
-            pass
+        self.hostNum = meta.get("hostNum")
         meta = self.meta()
         self.col.log("lmeta", meta)
         self.lmod = meta['mod']
@@ -148,12 +76,19 @@ class Syncer(object):
         if not self.col.basicCheck():
             self.col.log("basic check")
             return "basicCheckFailed"
-        # step 2: deletions
+        # step 2: startup and deletions
         runHook("sync", "meta")
-        lrem = self.removed()
-        rrem = self.server.start(
-            minUsn=self.minUsn, lnewer=self.lnewer, graves=lrem)
+        rrem = self.server.start(minUsn=self.minUsn, lnewer=self.lnewer)
+
+        # apply deletions to server
+        lgraves = self.removed()
+        while lgraves:
+            gchunk, lgraves = self._gravesChunk(lgraves)
+            self.server.applyGraves(chunk=gchunk)
+
+        # then apply server deletions here
         self.remove(rrem)
+
         # ...and small objects
         lchg = self.changes()
         rchg = self.server.applyChanges(changes=lchg)
@@ -183,7 +118,7 @@ class Syncer(object):
         if ret['status'] != "ok":
             # roll back and force full sync
             self.col.rollback()
-            self.col.modSchema()
+            self.col.modSchema(False)
             self.col.save()
             return "sanityCheckFailed"
         # finalize
@@ -192,6 +127,20 @@ class Syncer(object):
         self.finish(mod)
         return "success"
 
+    def _gravesChunk(self, graves):
+        lim = 250
+        chunk = dict(notes=[], cards=[], decks=[])
+        for cat in "notes", "cards", "decks":
+            if lim and graves[cat]:
+                chunk[cat] = graves[cat][:lim]
+                graves[cat] = graves[cat][lim:]
+                lim -= len(chunk[cat])
+
+        # anything remaining?
+        if graves['notes'] or graves['cards'] or graves['decks']:
+            return chunk, graves
+        return chunk, None
+
     def meta(self):
         return dict(
             mod=self.col.mod,
@@ -213,13 +162,6 @@ class Syncer(object):
             d['crt'] = self.col.crt
         return d
 
-    def applyChanges(self, changes):
-        self.rchg = changes
-        lchg = self.changes()
-        # merge our side before returning
-        self.mergeChanges(lchg, self.rchg)
-        return lchg
-
     def mergeChanges(self, lchg, rchg):
         # then the other objects
         self.mergeModels(rchg['models'])
@@ -247,14 +189,8 @@ class Syncer(object):
                 return "tag had usn = -1"
         found = False
         for m in self.col.models.all():
-            if self.col.server:
-                # the web upgrade was mistakenly setting usn
-                if m['usn'] < 0:
-                    m['usn'] = 0
-                    found = True
-            else:
-                if m['usn'] == -1:
-                    return "model had usn = -1"
+            if m['usn'] == -1:
+                return "model had usn = -1"
         if found:
             self.col.models.save()
         self.col.sched.reset()
@@ -272,22 +208,10 @@ class Syncer(object):
             len(self.col.decks.allConf()),
         ]
 
-    def sanityCheck2(self, client):
-        server = self.sanityCheck()
-        if client != server:
-            return dict(status="bad", c=client, s=server)
-        return dict(status="ok")
-
     def usnLim(self):
-        if self.col.server:
-            return "usn >= %d" % self.minUsn
-        else:
-            return "usn = -1"
+        return "usn = -1"
 
     def finish(self, mod=None):
-        if not mod:
-            # server side; we decide new mod time
-            mod = intTime(1000)
         self.col.ls = mod
         self.col._usn = self.maxUsn + 1
         # ensure we save the mod time even if no changes made
@@ -321,7 +245,7 @@ from notes where %s""" % d)
 
     def chunk(self):
         buf = dict(done=False)
-        lim = 2500
+        lim = 250
         while self.tablesLeft and lim:
             curTable = self.tablesLeft[0]
             if not self.cursor:
@@ -332,11 +256,10 @@ from notes where %s""" % d)
                 # table is empty
                 self.tablesLeft.pop(0)
                 self.cursor = None
-                # if we're the client, mark the objects as having been sent
-                if not self.col.server:
-                    self.col.db.execute(
-                        "update %s set usn=? where usn=-1"%curTable,
-                        self.maxUsn)
+                # mark the objects as having been sent
+                self.col.db.execute(
+                    "update %s set usn=? where usn=-1"%curTable,
+                    self.maxUsn)
             buf[curTable] = rows
             lim -= fetched
         if not self.tablesLeft:
@@ -358,12 +281,10 @@ from notes where %s""" % d)
         cards = []
         notes = []
         decks = []
-        if self.col.server:
-            curs = self.col.db.execute(
-                "select oid, type from graves where usn >= ?", self.minUsn)
-        else:
-            curs = self.col.db.execute(
-                "select oid, type from graves where usn = -1")
+
+        curs = self.col.db.execute(
+            "select oid, type from graves where usn = -1")
+
         for oid, type in curs:
             if type == REM_CARD:
                 cards.append(oid)
@@ -371,23 +292,16 @@ from notes where %s""" % d)
                 notes.append(oid)
             else:
                 decks.append(oid)
-        if not self.col.server:
-            self.col.db.execute("update graves set usn=? where usn=-1",
-                                 self.maxUsn)
-        return dict(cards=cards, notes=notes, decks=decks)
 
-    def start(self, minUsn, lnewer, graves):
-        self.maxUsn = self.col._usn
-        self.minUsn = minUsn
-        self.lnewer = not lnewer
-        lgraves = self.removed()
-        self.remove(graves)
-        return lgraves
+        self.col.db.execute("update graves set usn=? where usn=-1",
+                             self.maxUsn)
+
+        return dict(cards=cards, notes=notes, decks=decks)
 
     def remove(self, graves):
         # pretend to be the server so we don't set usn = -1
-        wasServer = self.col.server
         self.col.server = True
+
         # notes first, so we don't end up with duplicate graves
         self.col._remNotes(graves['notes'])
         # then cards
@@ -395,20 +309,18 @@ from notes where %s""" % d)
         # and decks
         for oid in graves['decks']:
             self.col.decks.rem(oid, childrenToo=False)
-        self.col.server = wasServer
+
+        self.col.server = False
 
     # Models
     ##########################################################################
 
     def getModels(self):
-        if self.col.server:
-            return [m for m in self.col.models.all() if m['usn'] >= self.minUsn]
-        else:
-            mods = [m for m in self.col.models.all() if m['usn'] == -1]
-            for m in mods:
-                m['usn'] = self.maxUsn
-            self.col.models.save()
-            return mods
+        mods = [m for m in self.col.models.all() if m['usn'] == -1]
+        for m in mods:
+            m['usn'] = self.maxUsn
+        self.col.models.save()
+        return mods
 
     def mergeModels(self, rchg):
         for r in rchg:
@@ -421,24 +333,22 @@ from notes where %s""" % d)
     ##########################################################################
 
     def getDecks(self):
-        if self.col.server:
-            return [
-                [g for g in self.col.decks.all() if g['usn'] >= self.minUsn],
-                [g for g in self.col.decks.allConf() if g['usn'] >= self.minUsn]
-            ]
-        else:
-            decks = [g for g in self.col.decks.all() if g['usn'] == -1]
-            for g in decks:
-                g['usn'] = self.maxUsn
-            dconf = [g for g in self.col.decks.allConf() if g['usn'] == -1]
-            for g in dconf:
-                g['usn'] = self.maxUsn
-            self.col.decks.save()
-            return [decks, dconf]
+        decks = [g for g in self.col.decks.all() if g['usn'] == -1]
+        for g in decks:
+            g['usn'] = self.maxUsn
+        dconf = [g for g in self.col.decks.allConf() if g['usn'] == -1]
+        for g in dconf:
+            g['usn'] = self.maxUsn
+        self.col.decks.save()
+        return [decks, dconf]
 
     def mergeDecks(self, rchg):
         for r in rchg[0]:
             l = self.col.decks.get(r['id'], False)
+            # work around mod time being stored as string
+            if l and not isinstance(l['mod'], int):
+                l['mod'] = int(l['mod'])
+
             # if missing locally or server is newer, update
             if not l or r['mod'] > l['mod']:
                 self.col.decks.update(r)
@@ -455,17 +365,13 @@ from notes where %s""" % d)
     ##########################################################################
 
     def getTags(self):
-        if self.col.server:
-            return [t for t, usn in self.col.tags.allItems()
-                    if usn >= self.minUsn]
-        else:
-            tags = []
-            for t, usn in self.col.tags.allItems():
-                if usn == -1:
-                    self.col.tags.tags[t] = self.maxUsn
-                    tags.append(t)
-            self.col.tags.save()
-            return tags
+        tags = []
+        for t, usn in self.col.tags.allItems():
+            if usn == -1:
+                self.col.tags.tags[t] = self.maxUsn
+                tags.append(t)
+        self.col.tags.save()
+        return tags
 
     def mergeTags(self, tags):
         self.col.tags.register(tags, usn=self.maxUsn)
@@ -514,35 +420,78 @@ from notes where %s""" % d)
     def mergeConf(self, conf):
         self.col.conf = conf
 
-# Local syncing for unit tests
+# Wrapper for requests that tracks upload/download progress
 ##########################################################################
 
-class LocalServer(Syncer):
+class AnkiRequestsClient:
 
-    # serialize/deserialize payload, so we don't end up sharing objects
-    # between cols
-    def applyChanges(self, changes):
-        l = json.loads; d = json.dumps
-        return l(d(Syncer.applyChanges(self, l(d(changes)))))
+    verify = True
+
+    def __init__(self):
+        self.session = requests.Session()
+
+    def post(self, url, data, headers):
+        data = _MonitoringFile(data)
+        headers['User-Agent'] = self._agentName()
+        return self.session.post(
+            url, data=data, headers=headers, stream=True, timeout=60, verify=self.verify)
+
+    def get(self, url, headers=None):
+        if headers is None:
+            headers = {}
+        headers['User-Agent'] = self._agentName()
+        return self.session.get(url, stream=True, timeout=60, verify=self.verify)
+
+    def streamContent(self, resp):
+        resp.raise_for_status()
+
+        buf = io.BytesIO()
+        for chunk in resp.iter_content(chunk_size=HTTP_BUF_SIZE):
+            runHook("httpRecv", len(chunk))
+            buf.write(chunk)
+        return buf.getvalue()
+
+    def _agentName(self):
+        from anki import version
+        return "Anki {}".format(version)
+
+# allow user to accept invalid certs in work/school settings
+if os.environ.get("ANKI_NOVERIFYSSL"):
+    AnkiRequestsClient.verify = False
+
+    import warnings
+    warnings.filterwarnings("ignore")
+
+class _MonitoringFile(io.BufferedReader):
+    def read(self, size=-1):
+        data = io.BufferedReader.read(self, HTTP_BUF_SIZE)
+        runHook("httpSend", len(data))
+        return data
 
 # HTTP syncing tools
 ##########################################################################
 
-# Calling code should catch the following codes:
-# - 501: client needs upgrade
-# - 502: ankiweb down
-# - 503/504: server too busy
+class HttpSyncer:
 
-class HttpSyncer(object):
-
-    def __init__(self, hkey=None, con=None):
+    def __init__(self, hkey=None, client=None, hostNum=None):
         self.hkey = hkey
         self.skey = checksum(str(random.random()))[:8]
-        self.con = con or httpCon()
+        self.client = client or AnkiRequestsClient()
+        self.postVars = {}
+        self.hostNum = hostNum
+        self.prefix = "sync/"
+
+    def syncURL(self):
+        if devMode:
+            url = "https://l1sync.ankiweb.net/"
+        else:
+            url = SYNC_BASE % (self.hostNum or "")
+        return url + self.prefix
 
     def assertOk(self, resp):
-        if resp['status'] != '200':
-            raise Exception("Unknown response code: %s" % resp['status'])
+        # not using raise_for_status() as aqt expects this error msg
+        if resp.status_code != 200:
+            raise Exception("Unknown response code: %s" % resp.status_code)
 
     # Posting data as a file
     ######################################################################
@@ -550,27 +499,23 @@ class HttpSyncer(object):
     # costly. We could send it as a raw post, but more HTTP clients seem to
     # support file uploading, so this is the more compatible choice.
 
-    def req(self, method, fobj=None, comp=6,
-                 badAuthRaises=True, hkey=True):
-        BOUNDARY="Anki-sync-boundary"
-        bdry = "--"+BOUNDARY
-        buf = StringIO()
-        # compression flag and session key as post vars
-        vars = {}
-        vars['c'] = 1 if comp else 0
-        if hkey:
-            vars['k'] = self.hkey
-            vars['s'] = self.skey
-        for (key, value) in vars.items():
-            buf.write(bdry + "\r\n")
+    def _buildPostData(self, fobj, comp):
+        BOUNDARY=b"Anki-sync-boundary"
+        bdry = b"--"+BOUNDARY
+        buf = io.BytesIO()
+        # post vars
+        self.postVars['c'] = 1 if comp else 0
+        for (key, value) in list(self.postVars.items()):
+            buf.write(bdry + b"\r\n")
             buf.write(
-                'Content-Disposition: form-data; name="%s"\r\n\r\n%s\r\n' %
-                (key, value))
+                ('Content-Disposition: form-data; name="%s"\r\n\r\n%s\r\n' %
+                (key, value)).encode("utf8"))
         # payload as raw data or json
+        rawSize = 0
         if fobj:
             # header
-            buf.write(bdry + "\r\n")
-            buf.write("""\
+            buf.write(bdry + b"\r\n")
+            buf.write(b"""\
 Content-Disposition: form-data; name="data"; filename="data"\r\n\
 Content-Type: application/octet-stream\r\n\r\n""")
             # write file into buffer, optionally compressing
@@ -584,53 +529,70 @@ Content-Type: application/octet-stream\r\n\r\n""")
                     if comp:
                         tgt.close()
                     break
+                rawSize += len(data)
                 tgt.write(data)
-            buf.write('\r\n' + bdry + '--\r\n')
+            buf.write(b"\r\n")
+        buf.write(bdry + b'--\r\n')
         size = buf.tell()
         # connection headers
         headers = {
-            'Content-Type': 'multipart/form-data; boundary=%s' % BOUNDARY,
+            'Content-Type': 'multipart/form-data; boundary=%s' % BOUNDARY.decode("utf8"),
             'Content-Length': str(size),
         }
-        body = buf.getvalue()
-        buf.close()
-        resp, cont = self.con.request(
-            SYNC_URL+method, "POST", headers=headers, body=body)
-        if not badAuthRaises:
-            # return false if bad auth instead of raising
-            if resp['status'] == '403':
-                return False
-        self.assertOk(resp)
-        return cont
+        buf.seek(0)
+
+        if size >= 100*1024*1024 or rawSize >= 250*1024*1024:
+            raise Exception("Collection too large to upload to AnkiWeb.")
+
+        return headers, buf
+
+    def req(self, method, fobj=None, comp=6, badAuthRaises=True):
+        headers, body = self._buildPostData(fobj, comp)
+
+        r = self.client.post(self.syncURL()+method, data=body, headers=headers)
+        if not badAuthRaises and r.status_code == 403:
+            return False
+        self.assertOk(r)
+
+        buf = self.client.streamContent(r)
+        return buf
 
 # Incremental sync over HTTP
 ######################################################################
 
 class RemoteServer(HttpSyncer):
 
-    def __init__(self, hkey):
-        HttpSyncer.__init__(self, hkey)
+    def __init__(self, hkey, hostNum):
+        HttpSyncer.__init__(self, hkey, hostNum=hostNum)
 
     def hostKey(self, user, pw):
         "Returns hkey or none if user/pw incorrect."
+        self.postVars = dict()
         ret = self.req(
-            "hostKey", StringIO(json.dumps(dict(u=user, p=pw))),
-            badAuthRaises=False, hkey=False)
-        if not ret:
-            # invalid auth
-            return
-        self.hkey = json.loads(ret)['key']
-        return self.hkey
-
-    def meta(self):
-        ret = self.req(
-            "meta", StringIO(json.dumps(dict(
-                v=SYNC_VER, cv="ankidesktop,%s,%s"%(anki.version, platDesc())))),
+            "hostKey", io.BytesIO(json.dumps(dict(u=user, p=pw)).encode("utf8")),
             badAuthRaises=False)
         if not ret:
             # invalid auth
             return
-        return json.loads(ret)
+        self.hkey = json.loads(ret.decode("utf8"))['key']
+        return self.hkey
+
+    def meta(self):
+        self.postVars = dict(
+            k=self.hkey,
+            s=self.skey,
+        )
+        ret = self.req(
+            "meta", io.BytesIO(json.dumps(dict(
+                v=SYNC_VER, cv="ankidesktop,%s,%s"%(anki.version, platDesc()))).encode("utf8")),
+            badAuthRaises=False)
+        if not ret:
+            # invalid auth
+            return
+        return json.loads(ret.decode("utf8"))
+
+    def applyGraves(self, **kw):
+        return self._run("applyGraves", kw)
 
     def applyChanges(self, **kw):
         return self._run("applyChanges", kw)
@@ -650,17 +612,24 @@ class RemoteServer(HttpSyncer):
     def finish(self, **kw):
         return self._run("finish", kw)
 
+    def abort(self, **kw):
+        return self._run("abort", kw)
+
     def _run(self, cmd, data):
         return json.loads(
-            self.req(cmd, StringIO(json.dumps(data))))
+            self.req(cmd, io.BytesIO(json.dumps(data).encode("utf8"))).decode("utf8"))
 
 # Full syncing
 ##########################################################################
 
 class FullSyncer(HttpSyncer):
 
-    def __init__(self, col, hkey, con):
-        HttpSyncer.__init__(self, hkey, con)
+    def __init__(self, col, hkey, client, hostNum):
+        HttpSyncer.__init__(self, hkey, client, hostNum=hostNum)
+        self.postVars = dict(
+            k=self.hkey,
+            v="ankidesktop,%s,%s"%(anki.version, platDesc()),
+        )
         self.col = col
 
     def download(self):
@@ -691,122 +660,205 @@ class FullSyncer(HttpSyncer):
             return False
         # apply some adjustments, then upload
         self.col.beforeUpload()
-        if self.req("upload", open(self.col.path, "rb")) != "OK":
+        if self.req("upload", open(self.col.path, "rb")) != b"OK":
             return False
         return True
 
 # Media syncing
 ##########################################################################
+#
+# About conflicts:
+# - to minimize data loss, if both sides are marked for sending and one
+#   side has been deleted, favour the add
+# - if added/changed on both sides, favour the server version on the
+#   assumption other syncers are in sync with the server
+#
 
-class MediaSyncer(object):
+class MediaSyncer:
 
     def __init__(self, col, server=None):
         self.col = col
         self.server = server
-        self.added = None
 
-    def sync(self, mediaUsn):
-        # step 1: check if there have been any changes
+    def sync(self):
+        # check if there have been any changes
         runHook("sync", "findMedia")
-        lusn = self.col.media.usn()
-        # if first sync or resync, clear list of files we think we've sent
-        if not lusn:
-            self.col.media.forceResync()
-        self.col.media.findChanges()
-        if lusn == mediaUsn and not self.col.media.hasChanged():
+        self.col.log("findChanges")
+        try:
+            self.col.media.findChanges()
+        except DBError:
+            return "corruptMediaDB"
+
+        # begin session and check if in sync
+        lastUsn = self.col.media.lastUsn()
+        ret = self.server.begin()
+        srvUsn = ret['usn']
+        if lastUsn == srvUsn and not self.col.media.haveDirty():
             return "noChanges"
-        # step 1.5: if resyncing, we need to get the list of files the server
-        # has and remove them from our local list of files to sync
-        if not lusn:
-            files = self.server.mediaList()
-            need = self.col.media.removeExisting(files)
-        else:
-            need = None
-        # step 2: send/recv deletions
-        runHook("sync", "removeMedia")
-        lrem = self.removed()
-        rrem = self.server.remove(fnames=lrem, minUsn=lusn)
-        self.remove(rrem)
-        # step 3: stream files from server
-        runHook("sync", "server")
-        while 1:
-            runHook("sync", "streamMedia")
-            usn = self.col.media.usn()
-            zip = self.server.files(minUsn=usn, need=need)
-            if self.addFiles(zip=zip):
+
+        # loop through and process changes from server
+        self.col.log("last local usn is %s"%lastUsn)
+        self.downloadCount = 0
+        while True:
+            data = self.server.mediaChanges(lastUsn=lastUsn)
+
+            self.col.log("mediaChanges resp count %d"%len(data))
+            if not data:
                 break
-        # step 4: stream files to the server
-        runHook("sync", "client")
-        while 1:
-            runHook("sync", "streamMedia")
-            zip, fnames = self.files()
+
+            need = []
+            lastUsn = data[-1][1]
+            for fname, rusn, rsum in data:
+                lsum, ldirty = self.col.media.syncInfo(fname)
+                self.col.log(
+                    "check: lsum=%s rsum=%s ldirty=%d rusn=%d fname=%s"%(
+                        (lsum and lsum[0:4]),
+                        (rsum and rsum[0:4]),
+                        ldirty,
+                        rusn,
+                        fname))
+
+                if rsum:
+                    # added/changed remotely
+                    if not lsum or lsum != rsum:
+                        self.col.log("will fetch")
+                        need.append(fname)
+                    else:
+                        self.col.log("have same already")
+                    ldirty and self.col.media.markClean([fname])
+                elif lsum:
+                    # deleted remotely
+                    if not ldirty:
+                        self.col.log("delete local")
+                        self.col.media.syncDelete(fname)
+                    else:
+                        # conflict; local add overrides remote delete
+                        self.col.log("conflict; will send")
+                else:
+                    # deleted both sides
+                    self.col.log("both sides deleted")
+                    ldirty and self.col.media.markClean([fname])
+
+            self._downloadFiles(need)
+
+            self.col.log("update last usn to %d"%lastUsn)
+            self.col.media.setLastUsn(lastUsn) # commits
+
+        # at this point we're all up to date with the server's changes,
+        # and we need to send our own
+
+        updateConflict = False
+        toSend = self.col.media.dirtyCount()
+        while True:
+            zip, fnames = self.col.media.mediaChangesZip()
             if not fnames:
-                # finished
                 break
-            usn = self.server.addFiles(zip=zip)
-            # after server has replied, safe to remove from log
-            self.col.media.forgetAdded(fnames)
-            self.col.media.setUsn(usn)
-        # step 5: sanity check during beta testing
-        # NOTE: when removing this, need to move server tidyup
-        # back from sanity check to addFiles
-        c = self.mediaSanity()
-        s = self.server.mediaSanity(client=c)
-        self.col.log("mediaSanity", c, s)
-        if c != s:
-            # if the sanity check failed, force a resync
+
+            runHook("syncMsg", ngettext(
+                "%d media change to upload", "%d media changes to upload", toSend)
+                    % toSend)
+
+            processedCnt, serverLastUsn = self.server.uploadChanges(zip)
+            self.col.media.markClean(fnames[0:processedCnt])
+
+            self.col.log("processed %d, serverUsn %d, clientUsn %d" % (
+                processedCnt, serverLastUsn, lastUsn
+            ))
+
+            if serverLastUsn - processedCnt == lastUsn:
+                self.col.log("lastUsn in sync, updating local")
+                lastUsn = serverLastUsn
+                self.col.media.setLastUsn(serverLastUsn) # commits
+            else:
+                self.col.log("concurrent update, skipping usn update")
+                # commit for markClean
+                self.col.media.db.commit()
+                updateConflict = True
+
+            toSend -= processedCnt
+
+        if updateConflict:
+            self.col.log("restart sync due to concurrent update")
+            return self.sync()
+
+        lcnt = self.col.media.mediaCount()
+        ret = self.server.mediaSanity(local=lcnt)
+        if ret == "OK":
+            return "OK"
+        else:
             self.col.media.forceResync()
-            return "sanityCheckFailed"
+            return ret
 
-    def removed(self):
-        return self.col.media.removed()
+    def _downloadFiles(self, fnames):
+        self.col.log("%d files to fetch"%len(fnames))
+        while fnames:
+            top = fnames[0:SYNC_ZIP_COUNT]
+            self.col.log("fetch %s"%top)
+            zipData = self.server.downloadFiles(files=top)
+            cnt = self.col.media.addFilesFromZip(zipData)
+            self.downloadCount += cnt
+            self.col.log("received %d files"%cnt)
+            fnames = fnames[cnt:]
 
-    def remove(self, fnames, minUsn=None):
-        self.col.media.syncRemove(fnames)
-        if minUsn is not None:
-            # we're the server
-            return self.col.media.removed()
-
-    def files(self):
-        return self.col.media.zipAdded()
-
-    def addFiles(self, zip):
-        "True if zip is the last in set. Server returns new usn instead."
-        return self.col.media.syncAdd(zip)
-
-    def mediaSanity(self):
-        return self.col.media.sanityCheck()
+            n = self.downloadCount
+            runHook("syncMsg", ngettext(
+                "%d media file downloaded", "%d media files downloaded", n)
+                    % n)
 
 # Remote media syncing
 ##########################################################################
 
 class RemoteMediaServer(HttpSyncer):
 
-    def __init__(self, hkey, con):
-        HttpSyncer.__init__(self, hkey, con)
+    def __init__(self, col, hkey, client, hostNum):
+        self.col = col
+        HttpSyncer.__init__(self, hkey, client, hostNum=hostNum)
+        self.prefix = "msync/"
 
-    def remove(self, **kw):
-        return json.loads(
-            self.req("remove", StringIO(json.dumps(kw))))
+    def begin(self):
+        self.postVars = dict(
+            k=self.hkey,
+            v="ankidesktop,%s,%s"%(anki.version, platDesc())
+        )
+        ret = self._dataOnly(self.req(
+            "begin", io.BytesIO(json.dumps(dict()).encode("utf8"))))
+        self.skey = ret['sk']
+        return ret
 
-    def files(self, **kw):
-        return self.req("files", StringIO(json.dumps(kw)))
+    # args: lastUsn
+    def mediaChanges(self, **kw):
+        self.postVars = dict(
+            sk=self.skey,
+        )
+        return self._dataOnly(
+            self.req("mediaChanges", io.BytesIO(json.dumps(kw).encode("utf8"))))
 
-    def addFiles(self, zip):
+    # args: files
+    def downloadFiles(self, **kw):
+        return self.req("downloadFiles", io.BytesIO(json.dumps(kw).encode("utf8")))
+
+    def uploadChanges(self, zip):
         # no compression, as we compress the zip file instead
-        return json.loads(
-            self.req("addFiles", StringIO(zip), comp=0))
+        return self._dataOnly(
+            self.req("uploadChanges", io.BytesIO(zip), comp=0))
 
+    # args: local
     def mediaSanity(self, **kw):
-        return json.loads(
-            self.req("mediaSanity", StringIO(json.dumps(kw))))
+        return self._dataOnly(
+            self.req("mediaSanity", io.BytesIO(json.dumps(kw).encode("utf8"))))
 
-    def mediaList(self):
-        return json.loads(
-            self.req("mediaList"))
+    def _dataOnly(self, resp):
+        resp = json.loads(resp.decode("utf8"))
+        if resp['err']:
+            self.col.log("error returned:%s"%resp['err'])
+            raise Exception("SyncError:%s"%resp['err'])
+        return resp['data']
 
     # only for unit tests
-    def mediatest(self, n):
-        return json.loads(
-            self.req("mediatest", StringIO(
-                json.dumps(dict(n=n)))))
+    def mediatest(self, cmd):
+        self.postVars = dict(
+            k=self.hkey,
+        )
+        return self._dataOnly(
+            self.req("newMediaTest", io.BytesIO(
+                json.dumps(dict(cmd=cmd)).encode("utf8"))))
