@@ -1,11 +1,13 @@
 # Copyright: Damien Elmes <anki@ichi2.net>
 # License: GNU AGPL, version 3 or later; http://www.gnu.org/licenses/agpl.html
+
+from anki import version as _version
+
 import getpass
-import os
 import sys
 import optparse
 import tempfile
-import __builtin__
+import builtins
 import locale
 import gettext
 
@@ -13,8 +15,7 @@ from aqt.qt import *
 import anki.lang
 from anki.consts import HELP_SITE
 from anki.lang import langDir
-from anki.utils import isMac
-from anki import version as _version
+from anki.utils import isMac, isLin
 
 appVersion=_version
 appWebsite="http://ankisrs.net/"
@@ -29,18 +30,18 @@ moduleDir = os.path.split(os.path.dirname(os.path.abspath(__file__)))[0]
 
 try:
     import aqt.forms
-except ImportError, e:
+except ImportError as e:
     if "forms" in str(e):
-        print "If you're running from git, did you run build_ui.sh?"
-        print
+        print("If you're running from git, did you run build_ui.sh?")
+        print()
     raise
 
 from anki.utils import checksum
 
 # Dialog manager - manages modeless windows
-##########################################################################
+##########################################################################emacs
 
-class DialogManager(object):
+class DialogManager:
 
     def __init__(self):
         from aqt import addcards, browser, editcurrent
@@ -53,7 +54,7 @@ class DialogManager(object):
     def open(self, name, *args):
         (creator, instance) = self._dialogs[name]
         if instance:
-            instance.setWindowState(Qt.WindowActive)
+            instance.setWindowState(Qt.WindowNoState)
             instance.activateWindow()
             instance.raise_()
             return instance
@@ -67,7 +68,7 @@ class DialogManager(object):
 
     def closeAll(self):
         "True if all closed successfully."
-        for (n, (creator, instance)) in self._dialogs.items():
+        for (n, (creator, instance)) in list(self._dialogs.items()):
             if instance:
                 if not instance.canClose():
                     return False
@@ -98,8 +99,8 @@ def setupLang(pm, app, force=None):
     # gettext
     _gtrans = gettext.translation(
         'anki', dir, languages=[lang], fallback=True)
-    __builtin__.__dict__['_'] = _gtrans.ugettext
-    __builtin__.__dict__['ngettext'] = _gtrans.ungettext
+    builtins.__dict__['_'] = _gtrans.gettext
+    builtins.__dict__['ngettext'] = _gtrans.ngettext
     anki.lang.setLang(lang, local=False)
     if lang in ("he","ar","fa"):
         app.setLayoutDirection(Qt.RightToLeft)
@@ -118,6 +119,8 @@ class AnkiApp(QApplication):
     # Single instance support on Win32/Linux
     ##################################################
 
+    appMsg = pyqtSignal(str)
+
     KEY = "anki"+checksum(getpass.getuser())
     TMOUT = 5000
 
@@ -133,14 +136,14 @@ class AnkiApp(QApplication):
         if args and args[0]:
             buf = os.path.abspath(args[0])
         if self.sendMsg(buf):
-            print "Already running; reusing existing instance."
+            print("Already running; reusing existing instance.")
             return True
         else:
             # send failed, so we're the first instance or the
             # previous instance died
             QLocalServer.removeServer(self.KEY)
             self._srv = QLocalServer(self)
-            self.connect(self._srv, SIGNAL("newConnection()"), self.onRecv)
+            self._srv.newConnection.connect(self.onRecv)
             self._srv.listen(self.KEY)
             return False
 
@@ -150,9 +153,10 @@ class AnkiApp(QApplication):
         if not sock.waitForConnected(self.TMOUT):
             # first instance or previous instance dead
             return False
-        sock.write(txt)
+        sock.write(txt.encode("utf8"))
         if not sock.waitForBytesWritten(self.TMOUT):
-            raise Exception("existing instance not emptying")
+            # existing instance running but hung
+            return False
         sock.disconnectFromServer()
         return True
 
@@ -161,9 +165,8 @@ class AnkiApp(QApplication):
         if not sock.waitForReadyRead(self.TMOUT):
             sys.stderr.write(sock.errorString())
             return
-        buf = sock.readAll()
-        buf = unicode(buf, sys.getfilesystemencoding(), "ignore")
-        self.emit(SIGNAL("appMsg"), buf)
+        path = bytes(sock.readAll()).decode("utf8")
+        self.appMsg.emit(path)
         sock.disconnectFromServer()
 
     # OS X file/url handler
@@ -171,7 +174,7 @@ class AnkiApp(QApplication):
 
     def event(self, evt):
         if evt.type() == QEvent.FileOpen:
-            self.emit(SIGNAL("appMsg"), evt.file() or "raise")
+            self.appMsg.emit(evt.file() or "raise")
             return True
         return QApplication.event(self, evt)
 
@@ -189,20 +192,27 @@ def parseArgs(argv):
     return parser.parse_args(argv[1:])
 
 def run():
+    try:
+        _run()
+    except Exception as e:
+        QMessageBox.critical(None, "Startup Error",
+                             "Please notify support of this error:\n\n"+
+                             traceback.format_exc())
+
+def _run():
     global mw
 
     # parse args
     opts, args = parseArgs(sys.argv)
-    opts.base = unicode(opts.base or "", sys.getfilesystemencoding())
-    opts.profile = unicode(opts.profile or "", sys.getfilesystemencoding())
+    opts.base = opts.base or ""
+    opts.profile = opts.profile or ""
 
     # on osx we'll need to add the qt plugins to the search path
-    if isMac and getattr(sys, 'frozen', None):
-        rd = os.path.abspath(moduleDir + "/../../..")
-        QCoreApplication.setLibraryPaths([rd])
 
-    if isMac:
-        QFont.insertSubstitution(".Lucida Grande UI", "Lucida Grande")
+    # work around pyqt loading wrong GL library
+    if isLin:
+        import ctypes
+        ctypes.CDLL('libGL.so.1', ctypes.RTLD_GLOBAL)
 
     # create the app
     app = AnkiApp(sys.argv)
@@ -225,13 +235,6 @@ No usable temporary folder found. Make sure C:\\temp exists or TEMP in your \
 environment points to a valid, writable folder.""")
         return
 
-    # qt version must be up to date
-    if qtmajor <= 4 and qtminor <= 6:
-        QMessageBox.warning(
-            None, "Error", "Your Qt version is known to be buggy. Until you "
-          "upgrade to a newer Qt, you may experience issues such as images "
-          "failing to show up during review.")
-
     # profile manager
     from aqt.profiles import ProfileManager
     pm = ProfileManager(opts.base, opts.profile)
@@ -246,6 +249,3 @@ environment points to a valid, writable folder.""")
     import aqt.main
     mw = aqt.main.AnkiQt(app, pm, args)
     app.exec_()
-
-if __name__ == "__main__":
-    run()
