@@ -16,6 +16,8 @@ from anki.hooks import addHook, remHook
 import aqt.forms
 import aqt.modelchooser
 import aqt.deckchooser
+from anki.lang import ngettext
+
 
 class ChangeMap(QDialog):
     def __init__(self, mw, model, current):
@@ -34,7 +36,7 @@ class ChangeMap(QDialog):
                 self.frm.fields.setCurrentRow(n)
             n += 1
         self.frm.fields.addItem(QListWidgetItem(_("Map to Tags")))
-        self.frm.fields.addItem(QListWidgetItem(_("Discard field")))
+        self.frm.fields.addItem(QListWidgetItem(_("Ignore field")))
         if not setCurrent:
             if current == "_tags":
                 self.frm.fields.setCurrentRow(n)
@@ -67,18 +69,20 @@ class ImportDialog(QDialog):
         self.importer = importer
         self.frm = aqt.forms.importing.Ui_ImportDialog()
         self.frm.setupUi(self)
-        self.connect(self.frm.buttonBox.button(QDialogButtonBox.Help),
-                     SIGNAL("clicked()"), self.helpRequested)
+        self.frm.buttonBox.button(QDialogButtonBox.Help).clicked.connect(
+            self.helpRequested)
         self.setupMappingFrame()
         self.setupOptions()
         self.modelChanged()
         self.frm.autoDetect.setVisible(self.importer.needDelimiter)
         addHook("currentModelChanged", self.modelChanged)
-        self.connect(self.frm.autoDetect, SIGNAL("clicked()"),
-                     self.onDelimiter)
+        self.frm.autoDetect.clicked.connect(self.onDelimiter)
         self.updateDelimiterButtonText()
         self.frm.allowHTML.setChecked(self.mw.pm.profile.get('allowHTML', True))
         self.frm.importMode.setCurrentIndex(self.mw.pm.profile.get('importMode', 1))
+        # import button
+        b = QPushButton(_("Import"))
+        self.frm.buttonBox.addButton(b, QDialogButtonBox.AcceptRole)
         self.exec_()
 
     def setupOptions(self):
@@ -87,8 +91,6 @@ class ImportDialog(QDialog):
             self.mw, self.frm.modelArea, label=False)
         self.deck = aqt.deckchooser.DeckChooser(
             self.mw, self.frm.deckArea, label=False)
-        self.connect(self.frm.importButton, SIGNAL("clicked()"),
-                     self.doImport)
 
     def modelChanged(self):
         self.importer.model = self.mw.col.models.current()
@@ -109,7 +111,11 @@ a tab, comma, and so on. If Anki is detecting the character incorrectly,
 you can enter it here. Use \\t to represent tab."""),
                 self, help="importing") or "\t"
         str = str.replace("\\t", "\t")
-        str = str.encode("ascii")
+        if len(str) > 1:
+            showWarning(_(
+                "Multi-character separators are not supported. "
+                "Please enter one character only."))
+            return
         self.hideMapping()
         def updateDelim():
             self.importer.delimiter = str
@@ -135,11 +141,11 @@ you can enter it here. Use \\t to represent tab."""),
         elif d == ":":
             d = _("Colon")
         else:
-            d = `d`
+            d = repr(d)
         txt = _("Fields separated by: %s") % d
         self.frm.autoDetect.setText(txt)
 
-    def doImport(self, update=False):
+    def accept(self):
         self.importer.mapping = self.mapping
         if not self.importer.mappingOk():
             showWarning(
@@ -161,7 +167,7 @@ you can enter it here. Use \\t to represent tab."""),
         except UnicodeDecodeError:
             showUnicodeWarning()
             return
-        except Exception, e:
+        except Exception as e:
             msg = _("Import failed.\n")
             err = repr(str(e))
             if "1-character string" in err:
@@ -169,7 +175,7 @@ you can enter it here. Use \\t to represent tab."""),
             elif "invalidTempFolder" in err:
                 msg += self.mw.errorHandler.tempFolderMsg()
             else:
-                msg += unicode(traceback.format_exc(), "ascii", "replace")
+                msg += str(traceback.format_exc(), "ascii", "replace")
             showText(msg)
             return
         finally:
@@ -208,7 +214,7 @@ you can enter it here. Use \\t to represent tab."""),
         self.mapbox.addWidget(self.mapwidget)
         self.grid = QGridLayout(self.mapwidget)
         self.mapwidget.setLayout(self.grid)
-        self.grid.setMargin(3)
+        self.grid.setContentsMargins(3,3,3,3)
         self.grid.setSpacing(6)
         fields = self.importer.fields()
         for num in range(len(self.mapping)):
@@ -223,8 +229,7 @@ you can enter it here. Use \\t to represent tab."""),
             self.grid.addWidget(QLabel(text), num, 1)
             button = QPushButton(_("Change"))
             self.grid.addWidget(button, num, 2)
-            self.connect(button, SIGNAL("clicked()"),
-                         lambda s=self,n=num: s.changeMappingNum(n))
+            button.clicked.connect(lambda _, s=self,n=num: s.changeMappingNum(n))
 
     def changeMappingNum(self, n):
         f = ChangeMap(self.mw, self.importer.model, self.mapping[n]).getField()
@@ -249,7 +254,7 @@ you can enter it here. Use \\t to represent tab."""),
         QDialog.reject(self)
 
     def helpRequested(self):
-        openHelp("FileImport")
+        openHelp("importing")
 
 
 def showUnicodeWarning():
@@ -265,7 +270,7 @@ def onImport(mw):
                    filter=filt)
     if not file:
         return
-    file = unicode(file)
+    file = str(file)
     importFile(mw, file)
 
 def importFile(mw, file):
@@ -292,9 +297,9 @@ def importFile(mw, file):
         except UnicodeDecodeError:
             showUnicodeWarning()
             return
-        except Exception, e:
+        except Exception as e:
             msg = repr(str(e))
-            if msg == "unknownFormat":
+            if msg == "'unknownFormat'":
                 if file.endswith(".anki2"):
                     showWarning(_("""\
 .anki2 files are not designed for importing. If you're trying to restore from a \
@@ -303,7 +308,7 @@ backup, please see the 'Backups' section of the user manual."""))
                     showWarning(_("Unknown file format."))
             else:
                 msg = _("Import failed. Debugging info:\n")
-                msg += unicode(traceback.format_exc(), "ascii", "replace")
+                msg += str(traceback.format_exc())
                 showText(msg)
             return
         finally:
@@ -312,11 +317,11 @@ backup, please see the 'Backups' section of the user manual."""))
     else:
         # if it's an apkg/zip, first test it's a valid file
         if importer.__class__.__name__ == "AnkiPackageImporter":
-            z = zipfile.ZipFile(importer.file)
             try:
+                z = zipfile.ZipFile(importer.file)
                 z.getinfo("collection.anki2")
             except:
-                showWarning(_("The provided file is not a valid .apkg file."))
+                showWarning(invalidZipMsg())
                 return
             # we need to ask whether to import/replace
             if not setupApkgImport(mw, importer):
@@ -325,13 +330,8 @@ backup, please see the 'Backups' section of the user manual."""))
         try:
             importer.run()
         except zipfile.BadZipfile:
-            msg = _("""\
-This file does not appear to be a valid .apkg file. If you're getting this \
-error from a file downloaded from AnkiWeb, chances are that your download \
-failed. Please try again, and if the problem persists, please try again \
-with a different browser.""")
-            showWarning(msg)
-        except Exception, e:
+            showWarning(invalidZipMsg())
+        except Exception as e:
             err = repr(str(e))
             if "invalidFile" in err:
                 msg = _("""\
@@ -344,7 +344,7 @@ Invalid file. Please restore from backup.""")
 Unable to import from a read-only file."""))
             else:
                 msg = _("Import failed.\n")
-                msg += unicode(traceback.format_exc(), "ascii", "replace")
+                msg += str(traceback.format_exc())
                 showText(msg)
         else:
             log = "\n".join(importer.log)
@@ -356,6 +356,13 @@ Unable to import from a read-only file."""))
             mw.progress.finish()
         mw.reset()
 
+def invalidZipMsg():
+    return _("""\
+This file does not appear to be a valid .apkg file. If you're getting this \
+error from a file downloaded from AnkiWeb, chances are that your download \
+failed. Please try again, and if the problem persists, please try again \
+with a different browser.""")
+
 def setupApkgImport(mw, importer):
     base = os.path.basename(importer.file).lower()
     full = (base == "collection.apkg") or re.match("backup-.*\\.apkg", base)
@@ -378,14 +385,27 @@ def replaceWithApkg(mw, file, backup):
     mw.unloadCollection()
     # overwrite collection
     z = zipfile.ZipFile(file)
-    z.extract("collection.anki2", mw.pm.profileFolder())
+    try:
+        z.extract("collection.anki2", mw.pm.profileFolder())
+    except:
+        showWarning(_("The provided file is not a valid .apkg file."))
+        return
     # because users don't have a backup of media, it's safer to import new
     # data and rely on them running a media db check to get rid of any
     # unwanted media. in the future we might also want to deduplicate this
     # step
     d = os.path.join(mw.pm.profileFolder(), "collection.media")
-    for c, file in json.loads(z.read("media")).items():
-        open(os.path.join(d, file), "wb").write(z.read(str(c)))
+    for n, (cStr, file) in enumerate(
+            json.loads(z.read("media").decode("utf8")).items()):
+        mw.progress.update(ngettext("Processed %d media file",
+                                    "Processed %d media files", n) % n)
+        size = z.getinfo(cStr).file_size
+        dest = os.path.join(d, file)
+        # if we have a matching file size
+        if os.path.exists(dest) and size == os.stat(dest).st_size:
+            continue
+        data = z.read(cStr)
+        open(dest, "wb").write(data)
     z.close()
     # reload
     mw.loadCollection()
