@@ -4,6 +4,7 @@
 
 import datetime, time
 from aqt.qt import *
+import anki.lang
 from aqt.utils import openFolder, showWarning, getText, openHelp, showInfo
 import aqt
 
@@ -18,8 +19,11 @@ class Preferences(QDialog):
         self.prof = self.mw.pm.profile
         self.form = aqt.forms.preferences.Ui_Preferences()
         self.form.setupUi(self)
-        self.connect(self.form.buttonBox, SIGNAL("helpRequested()"),
-                     lambda: openHelp("profileprefs"))
+        self.form.buttonBox.button(QDialogButtonBox.Help).setAutoDefault(False)
+        self.form.buttonBox.button(QDialogButtonBox.Close).setAutoDefault(False)
+        self.form.buttonBox.helpRequested.connect(lambda: openHelp("profileprefs"))
+        self.silentlyClose = True
+        self.setupLang()
         self.setupCollection()
         self.setupNetwork()
         self.setupBackup()
@@ -27,6 +31,9 @@ class Preferences(QDialog):
         self.show()
 
     def accept(self):
+        # avoid exception if main window is already closed
+        if not self.mw.col:
+            return
         self.updateCollection()
         self.updateNetwork()
         self.updateBackup()
@@ -34,10 +41,32 @@ class Preferences(QDialog):
         self.mw.pm.save()
         self.mw.reset()
         self.done(0)
+        aqt.dialogs.markClosed("Preferences")
 
     def reject(self):
         self.accept()
 
+    # Language
+    ######################################################################
+
+    def setupLang(self):
+        f = self.form
+        f.lang.addItems([x[0] for x in anki.lang.langs])
+        f.lang.setCurrentIndex(self.langIdx())
+        f.lang.currentIndexChanged.connect(self.onLangIdxChanged)
+
+    def langIdx(self):
+        codes = [x[1] for x in anki.lang.langs]
+        try:
+            return codes.index(anki.lang.getLang())
+        except:
+            return codes.index("en")
+
+    def onLangIdxChanged(self, idx):
+        code = anki.lang.langs[idx][1]
+        self.mw.pm.setLang(code)
+        showInfo(_("Please restart Anki to complete language change."), parent=self)
+
     # Collection options
     ######################################################################
 
@@ -45,15 +74,18 @@ class Preferences(QDialog):
         import anki.consts as c
         f = self.form
         qc = self.mw.col.conf
-        self.startDate = datetime.datetime.fromtimestamp(self.mw.col.crt)
-        f.dayOffset.setValue(self.startDate.hour)
+        self._setupDayCutoff()
         f.lrnCutoff.setValue(qc['collapseTime']/60.0)
         f.timeLimit.setValue(qc['timeLim']/60.0)
         f.showEstimates.setChecked(qc['estTimes'])
         f.showProgress.setChecked(qc['dueCounts'])
-        f.newSpread.addItems(c.newCardSchedulingLabels().values())
+        f.nightMode.setChecked(qc.get("nightMode", False))
+        f.newSpread.addItems(list(c.newCardSchedulingLabels().values()))
         f.newSpread.setCurrentIndex(qc['newSpread'])
         f.useCurrent.setCurrentIndex(int(not qc.get("addToCur", True)))
+        f.dayLearnFirst.setChecked(qc.get("dayLearnFirst", False))
+        if self.mw.col.schedVer() != 2:
+            f.dayLearnFirst.setVisible(False)
 
     def updateCollection(self):
         f = self.form
@@ -62,15 +94,45 @@ class Preferences(QDialog):
         qc['dueCounts'] = f.showProgress.isChecked()
         qc['estTimes'] = f.showEstimates.isChecked()
         qc['newSpread'] = f.newSpread.currentIndex()
+        qc['nightMode'] = f.nightMode.isChecked()
         qc['timeLim'] = f.timeLimit.value()*60
         qc['collapseTime'] = f.lrnCutoff.value()*60
         qc['addToCur'] = not f.useCurrent.currentIndex()
-        hrs = f.dayOffset.value()
+        qc['dayLearnFirst'] = f.dayLearnFirst.isChecked()
+        self._updateDayCutoff()
+        d.setMod()
+
+    # Day cutoff
+    ######################################################################
+
+    def _setupDayCutoff(self):
+        if self.mw.col.schedVer() == 2:
+            self._setupDayCutoffV2()
+        else:
+            self._setupDayCutoffV1()
+
+    def _setupDayCutoffV1(self):
+        self.startDate = datetime.datetime.fromtimestamp(self.mw.col.crt)
+        self.form.dayOffset.setValue(self.startDate.hour)
+
+    def _setupDayCutoffV2(self):
+        self.form.dayOffset.setValue(self.mw.col.conf.get("rollover", 4))
+
+    def _updateDayCutoff(self):
+        if self.mw.col.schedVer() == 2:
+            self._updateDayCutoffV2()
+        else:
+            self._updateDayCutoffV1()
+
+    def _updateDayCutoffV1(self):
+        hrs = self.form.dayOffset.value()
         old = self.startDate
         date = datetime.datetime(
             old.year, old.month, old.day, hrs)
-        d.crt = int(time.mktime(date.timetuple()))
-        d.setMod()
+        self.mw.col.crt = int(time.mktime(date.timetuple()))
+
+    def _updateDayCutoffV2(self):
+        self.mw.col.conf['rollover'] = self.form.dayOffset.value()
 
     # Network
     ######################################################################
@@ -84,8 +146,7 @@ class Preferences(QDialog):
             self._hideAuth()
         else:
             self.form.syncUser.setText(self.prof.get('syncUser', ""))
-            self.connect(self.form.syncDeauth, SIGNAL("clicked()"),
-                         self.onSyncDeauth)
+            self.form.syncDeauth.clicked.connect(self.onSyncDeauth)
 
     def _hideAuth(self):
         self.form.syncDeauth.setVisible(False)
@@ -96,59 +157,35 @@ Not currently enabled; click the sync button in the main window to enable."""))
 
     def onSyncDeauth(self):
         self.prof['syncKey'] = None
+        self.mw.col.media.forceResync()
         self._hideAuth()
 
     def updateNetwork(self):
         self.prof['autoSync'] = self.form.syncOnProgramOpen.isChecked()
         self.prof['syncMedia'] = self.form.syncMedia.isChecked()
         if self.form.fullSync.isChecked():
-            self.mw.hideSchemaMsg = True
-            self.mw.col.modSchema()
+            self.mw.col.modSchema(check=False)
             self.mw.col.setMod()
-            self.mw.hideSchemaMsg = False
 
     # Backup
     ######################################################################
 
     def setupBackup(self):
         self.form.numBackups.setValue(self.prof['numBackups'])
-        self.form.compressBackups.setChecked(self.prof.get("compressBackups", True))
-        self.connect(self.form.openBackupFolder,
-                     SIGNAL("linkActivated(QString)"),
-                     self.onOpenBackup)
+        self.form.openBackupFolder.linkActivated.connect(self.onOpenBackup)
 
     def onOpenBackup(self):
         openFolder(self.mw.pm.backupFolder())
 
     def updateBackup(self):
         self.prof['numBackups'] = self.form.numBackups.value()
-        self.prof['compressBackups'] = self.form.compressBackups.isChecked()
 
     # Basic & Advanced Options
     ######################################################################
 
     def setupOptions(self):
-        self.form.stripHTML.setChecked(self.prof['stripHTML'])
         self.form.pastePNG.setChecked(self.prof.get("pastePNG", False))
-        self.connect(
-            self.form.profilePass, SIGNAL("clicked()"),
-            self.onProfilePass)
 
     def updateOptions(self):
-        self.prof['stripHTML'] = self.form.stripHTML.isChecked()
         self.prof['pastePNG'] = self.form.pastePNG.isChecked()
 
-    def onProfilePass(self):
-        pw, ret = getText(_("""\
-Lock account with password, or leave blank:"""))
-        if not ret:
-            return
-        if not pw:
-            self.prof['key'] = None
-            return
-        pw2, ret = getText(_("Confirm password:"))
-        if not ret:
-            return
-        if pw != pw2:
-            showWarning(_("Passwords didn't match"))
-        self.prof['key'] = self.mw.pm._pwhash(pw)
