@@ -8,20 +8,21 @@
 
 import os
 import random
-import cPickle
+import pickle
+import shutil
+import io
 import locale
 import re
 
 from aqt.qt import *
 from anki.db import DB
 from anki.utils import isMac, isWin, intTime, checksum
-from anki.lang import langs
+import anki.lang
 from aqt.utils import showWarning
 from aqt import appHelpSite
 import aqt.forms
 from send2trash import send2trash
 
-
 metaConf = dict(
     ver=0,
     updates=True,
@@ -36,10 +37,9 @@ metaConf = dict(
 
 profileConf = dict(
     # profile
-    key=None,
     mainWindowGeom=None,
     mainWindowState=None,
-    numBackups=30,
+    numBackups=50,
     lastOptimize=intTime(),
     # editing
     fullSearch=False,
@@ -59,20 +59,20 @@ profileConf = dict(
     importMode=1,
 )
 
-class ProfileManager(object):
+class ProfileManager:
 
     def __init__(self, base=None, profile=None):
         self.name = None
+        self.db = None
         # instantiate base folder
-        if base:
-            self.base = os.path.abspath(base)
-        else:
-            self.base = self._defaultBase()
-        self.ensureBaseExists()
+        self._setBaseFolder(base)
         # load metadata
         self.firstRun = self._loadMeta()
         # did the user request a profile to start up with?
         if profile:
+            if profile not in self.profiles():
+                QMessageBox.critical(None, "Error", "Requested profile does not exist.")
+                sys.exit(1)
             try:
                 self.load(profile)
             except TypeError:
@@ -88,67 +88,149 @@ class ProfileManager(object):
             # can't translate, as lang not initialized
             QMessageBox.critical(
                 None, "Error", """\
-Anki can't write to the harddisk. Please see the \
-documentation for information on using a flash drive.""")
+Anki could not create the folder %s. Please ensure that location is not \
+read-only and you have permission to write to it. If you cannot fix this \
+issue, please see the documentation for information on running Anki from \
+a flash drive.""" % self.base)
             raise
 
+    # Folder migration
+    ######################################################################
+
+    def _oldFolderLocation(self):
+        if isMac:
+            return os.path.expanduser("~/Documents/Anki")
+        elif isWin:
+            loc = QStandardPaths.writableLocation(QStandardPaths.DocumentsLocation)
+            return os.path.join(loc, "Anki")
+        else:
+            p = os.path.expanduser("~/Anki")
+            if os.path.isdir(p):
+                return p
+            else:
+                loc = QStandardPaths.writableLocation(QStandardPaths.DocumentsLocation)
+                if loc[:-1] == QStandardPaths.writableLocation(
+                        QStandardPaths.HomeLocation):
+                    # occasionally "documentsLocation" will return the home
+                    # folder because the Documents folder isn't configured
+                    # properly; fall back to an English path
+                    return os.path.expanduser("~/Documents/Anki")
+                else:
+                    return os.path.join(loc, "Anki")
+
+    def maybeMigrateFolder(self):
+        oldBase = self._oldFolderLocation()
+
+        if not os.path.exists(self.base) and os.path.isdir(oldBase):
+            shutil.move(oldBase, self.base)
+
     # Profile load/save
     ######################################################################
 
     def profiles(self):
-        return sorted(
-            unicode(x, "utf8") for x in
+        return sorted(x for x in
             self.db.list("select name from profiles")
             if x != "_global")
 
-    def load(self, name, passwd=None):
-        prof = cPickle.loads(
-            self.db.scalar("select data from profiles where name = ?",
-                           name.encode("utf8")))
-        if prof['key'] and prof['key'] != self._pwhash(passwd):
-            self.name = None
-            return False
-        if name != "_global":
-            self.name = name
-            self.profile = prof
+    def _unpickle(self, data):
+        class Unpickler(pickle.Unpickler):
+            def find_class(self, module, name):
+                fn = super().find_class(module, name)
+                if module == "sip" and name == "_unpickle_type":
+                    def wrapper(mod, obj, args):
+                        if mod.startswith("PyQt4") and obj == "QByteArray":
+                            # can't trust str objects from python 2
+                            return QByteArray()
+                        return fn(mod, obj, args)
+                    return wrapper
+                else:
+                    return fn
+        up = Unpickler(io.BytesIO(data), errors="ignore")
+        return up.load()
+
+    def _pickle(self, obj):
+        return pickle.dumps(obj, protocol=0)
+
+    def load(self, name):
+        assert name != "_global"
+        data = self.db.scalar("select cast(data as blob) from profiles where name = ?", name)
+        self.name = name
+        try:
+            self.profile = self._unpickle(data)
+        except:
+            print("resetting corrupt profile")
+            self.profile = profileConf.copy()
+            self.save()
         return True
 
     def save(self):
         sql = "update profiles set data = ? where name = ?"
-        self.db.execute(sql, cPickle.dumps(self.profile),
-                        self.name.encode("utf8"))
-        self.db.execute(sql, cPickle.dumps(self.meta), "_global")
+        self.db.execute(sql, self._pickle(self.profile), self.name)
+        self.db.execute(sql, self._pickle(self.meta), "_global")
         self.db.commit()
 
     def create(self, name):
         prof = profileConf.copy()
-        self.db.execute("insert into profiles values (?, ?)",
-                        name.encode("utf8"), cPickle.dumps(prof))
+        self.db.execute("insert or ignore into profiles values (?, ?)",
+                        name, self._pickle(prof))
         self.db.commit()
 
     def remove(self, name):
         p = self.profileFolder()
         if os.path.exists(p):
             send2trash(p)
-        self.db.execute("delete from profiles where name = ?",
-                        name.encode("utf8"))
+        self.db.execute("delete from profiles where name = ?", name)
         self.db.commit()
 
+    def trashCollection(self):
+        p = self.collectionPath()
+        if os.path.exists(p):
+            send2trash(p)
+
     def rename(self, name):
         oldName = self.name
         oldFolder = self.profileFolder()
         self.name = name
         newFolder = self.profileFolder(create=False)
         if os.path.exists(newFolder):
-            showWarning(_("Folder already exists."))
-            self.name = oldName
-            return
+            if (oldFolder != newFolder) and (
+                    oldFolder.lower() == newFolder.lower()):
+                # OS is telling us the folder exists because it does not take
+                # case into account; use a temporary folder location
+                midFolder = ''.join([oldFolder, '-temp'])
+                if not os.path.exists(midFolder):
+                    os.rename(oldFolder, midFolder)
+                    oldFolder = midFolder
+                else:
+                    showWarning(_("Please remove the folder %s and try again.")
+                            % midFolder)
+                    self.name = oldName
+                    return
+            else:
+                showWarning(_("Folder already exists."))
+                self.name = oldName
+                return
+
         # update name
         self.db.execute("update profiles set name = ? where name = ?",
-                        name.encode("utf8"), oldName.encode("utf-8"))
+                        name, oldName)
         # rename folder
-        os.rename(oldFolder, newFolder)
-        self.db.commit()
+        try:
+            os.rename(oldFolder, newFolder)
+        except WindowsError as e:
+            self.db.rollback()
+            if "Access is denied" in e:
+                showWarning(_("""\
+Anki could not rename your profile because it could not rename the profile \
+folder on disk. Please ensure you have permission to write to Documents/Anki \
+and no other programs are accessing your profile folders, then try again."""))
+            else:
+                raise
+        except:
+            self.db.rollback()
+            raise
+        else:
+            self.db.commit()
 
     # Folder handling
     ######################################################################
@@ -160,7 +242,7 @@ documentation for information on using a flash drive.""")
         return path
 
     def addonFolder(self):
-        return self._ensureExists(os.path.join(self.base, "addons"))
+        return self._ensureExists(os.path.join(self.base, "addons21"))
 
     def backupFolder(self):
         return self._ensureExists(
@@ -177,50 +259,77 @@ documentation for information on using a flash drive.""")
             os.makedirs(path)
         return path
 
+    def _setBaseFolder(self, cmdlineBase):
+        if cmdlineBase:
+            self.base = os.path.abspath(cmdlineBase)
+        elif os.environ.get("ANKI_BASE"):
+            self.base = os.path.abspath(os.environ["ANKI_BASE"])
+        else:
+            self.base = self._defaultBase()
+            self.maybeMigrateFolder()
+        self.ensureBaseExists()
+
     def _defaultBase(self):
         if isWin:
-            if False: #qtmajor >= 5:
-                loc = QStandardPaths.writeableLocation(QStandardPaths.DocumentsLocation)
-            else:
-                loc = QDesktopServices.storageLocation(QDesktopServices.DocumentsLocation)
-            return os.path.join(loc, "Anki")
+            loc = QStandardPaths.writableLocation(QStandardPaths.AppDataLocation)
+            # the returned value seem to automatically include the app name, but we use Anki2 rather
+            # than Anki
+            assert loc.endswith("/Anki")
+            loc += "2"
+            return loc
         elif isMac:
-            return os.path.expanduser("~/Documents/Anki")
+            return os.path.expanduser("~/Library/Application Support/Anki2")
         else:
-            return os.path.expanduser("~/Anki")
+            dataDir = os.environ.get(
+                "XDG_DATA_HOME", os.path.expanduser("~/.local/share"))
+            if not os.path.exists(dataDir):
+                os.makedirs(dataDir)
+            return os.path.join(dataDir, "Anki2")
 
     def _loadMeta(self):
-        path = os.path.join(self.base, "prefs.db")
+        opath = os.path.join(self.base, "prefs.db")
+        path = os.path.join(self.base, "prefs21.db")
+        if os.path.exists(opath) and not os.path.exists(path):
+            shutil.copy(opath, path)
+
         new = not os.path.exists(path)
         def recover():
             # if we can't load profile, start with a new one
-            os.rename(path, path+".broken")
+            if self.db:
+                try:
+                    self.db.close()
+                except:
+                    pass
+            for suffix in ("", "-journal"):
+                fpath = path + suffix
+                if os.path.exists(fpath):
+                    os.unlink(fpath)
             QMessageBox.warning(
                 None, "Preferences Corrupt", """\
-Anki's prefs.db file was corrupt and has been recreated. If you were using multiple \
+Anki's prefs21.db file was corrupt and has been recreated. If you were using multiple \
 profiles, please add them back using the same names to recover your cards.""")
         try:
-            self.db = DB(path, text=str)
+            self.db = DB(path)
+            assert self.db.scalar("pragma integrity_check") == "ok"
             self.db.execute("""
 create table if not exists profiles
 (name text primary key, data text not null);""")
+            data = self.db.scalar(
+                "select cast(data as blob) from profiles where name = '_global'")
         except:
             recover()
             return self._loadMeta()
         if not new:
-            # load previously created
+            # load previously created data
             try:
-                self.meta = cPickle.loads(
-                    self.db.scalar(
-                        "select data from profiles where name = '_global'"))
+                self.meta = self._unpickle(data)
                 return
             except:
-                recover()
-                return self._loadMeta()
+                print("resetting corrupt _global")
         # create a default global profile
         self.meta = metaConf.copy()
         self.db.execute("insert or replace into profiles values ('_global', ?)",
-                        cPickle.dumps(metaConf))
+                        self._pickle(metaConf))
         self._setDefaultLang()
         return True
 
@@ -229,16 +338,13 @@ create table if not exists profiles
         if self.firstRun:
             self.create(_("User 1"))
             p = os.path.join(self.base, "README.txt")
-            open(p, "w").write((_("""\
+            open(p, "w").write(_("""\
 This folder stores all of your Anki data in a single location,
 to make backups easy. To tell Anki to use a different location,
 please see:
 
 %s
-""") % (appHelpSite +  "#startupopts")).encode("utf8"))
-
-    def _pwhash(self, passwd):
-        return checksum(unicode(self.meta['id'])+unicode(passwd))
+""") % (appHelpSite +  "#startupopts"))
 
     # Default language
     ######################################################################
@@ -247,8 +353,8 @@ please see:
     def _setDefaultLang(self):
         # the dialog expects _ to be defined, but we're running before
         # setupLang() has been called. so we create a dummy op for now
-        import __builtin__
-        __builtin__.__dict__['_'] = lambda x: x
+        import builtins
+        builtins.__dict__['_'] = lambda x: x
         # create dialog
         class NoCloseDiag(QDialog):
             def reject(self):
@@ -256,8 +362,8 @@ please see:
         d = self.langDiag = NoCloseDiag()
         f = self.langForm = aqt.forms.setlang.Ui_Dialog()
         f.setupUi(d)
-        d.connect(d, SIGNAL("accepted()"), self._onLangSelected)
-        d.connect(d, SIGNAL("rejected()"), lambda: True)
+        d.accepted.connect(self._onLangSelected)
+        d.rejected.connect(lambda: True)
         # default to the system language
         try:
             (lang, enc) = locale.getdefaultlocale()
@@ -269,7 +375,7 @@ please see:
         # find index
         idx = None
         en = None
-        for c, (name, code) in enumerate(langs):
+        for c, (name, code) in enumerate(anki.lang.langs):
             if code == "en":
                 en = c
             if code == lang:
@@ -278,13 +384,13 @@ please see:
         if idx is None:
             idx = en
         # update list
-        f.lang.addItems([x[0] for x in langs])
+        f.lang.addItems([x[0] for x in anki.lang.langs])
         f.lang.setCurrentRow(idx)
         d.exec_()
 
     def _onLangSelected(self):
         f = self.langForm
-        obj = langs[f.lang.currentRow()]
+        obj = anki.lang.langs[f.lang.currentRow()]
         code = obj[1]
         name = obj[0]
         en = "Are you sure you wish to display Anki's interface in %s?"
@@ -293,7 +399,11 @@ please see:
             QMessageBox.No)
         if r != QMessageBox.Yes:
             return self._setDefaultLang()
+        self.setLang(code)
+
+    def setLang(self, code):
         self.meta['defaultLang'] = code
         sql = "update profiles set data = ? where name = ?"
-        self.db.execute(sql, cPickle.dumps(self.meta), "_global")
+        self.db.execute(sql, self._pickle(self.meta), "_global")
         self.db.commit()
+        anki.lang.setLang(code, local=False)
