@@ -1,11 +1,20 @@
 # Copyright: Damien Elmes <anki@ichi2.net>
+# -*- coding: utf-8 -*-
 # License: GNU AGPL, version 3 or later; http://www.gnu.org/licenses/agpl.html
 
 from aqt.qt import *
-import re, os, sys, urllib, subprocess
+import re, os, sys, urllib.request, urllib.parse, urllib.error, subprocess
 import aqt
 from anki.sound import stripSounds
 from anki.utils import isWin, isMac, invalidFilename
+from contextlib import contextmanager
+
+@contextmanager
+def noBundledLibs():
+    oldlpath = os.environ.pop("LD_LIBRARY_PATH", None)
+    yield
+    if oldlpath is not None:
+        os.environ["LD_LIBRARY_PATH"] = oldlpath
 
 def openHelp(section):
     link = aqt.appHelpSite
@@ -15,17 +24,18 @@ def openHelp(section):
 
 def openLink(link):
     tooltip(_("Loading..."), period=1000)
-    QDesktopServices.openUrl(QUrl(link))
+    with noBundledLibs():
+        QDesktopServices.openUrl(QUrl(link))
 
-def showWarning(text, parent=None, help=""):
+def showWarning(text, parent=None, help="", title="Anki"):
     "Show a small warning with an OK button."
-    return showInfo(text, parent, help, "warning")
+    return showInfo(text, parent, help, "warning", title=title)
 
-def showCritical(text, parent=None, help=""):
+def showCritical(text, parent=None, help="", title="Anki"):
     "Show a small critical error with an OK button."
-    return showInfo(text, parent, help, "critical")
+    return showInfo(text, parent, help, "critical", title=title)
 
-def showInfo(text, parent=False, help="", type="info"):
+def showInfo(text, parent=False, help="", type="info", title="Anki"):
     "Show a small info window with an OK button."
     if parent is False:
         parent = aqt.mw.app.activeWindow() or aqt.mw
@@ -39,23 +49,25 @@ def showInfo(text, parent=False, help="", type="info"):
     mb.setText(text)
     mb.setIcon(icon)
     mb.setWindowModality(Qt.WindowModal)
+    mb.setWindowTitle(title)
     b = mb.addButton(QMessageBox.Ok)
     b.setDefault(True)
     if help:
         b = mb.addButton(QMessageBox.Help)
-        b.connect(b, SIGNAL("clicked()"), lambda: openHelp(help))
+        b.clicked.connect(lambda: openHelp(help))
         b.setAutoDefault(False)
     return mb.exec_()
 
-def showText(txt, parent=None, type="text", run=True):
+def showText(txt, parent=None, type="text", run=True, geomKey=None, \
+        minWidth=500, minHeight=400, title="Anki"):
     if not parent:
         parent = aqt.mw.app.activeWindow() or aqt.mw
     diag = QDialog(parent)
-    diag.setWindowTitle("Anki")
+    diag.setWindowTitle(title)
     layout = QVBoxLayout(diag)
     diag.setLayout(layout)
-    text = QTextEdit()
-    text.setReadOnly(True)
+    text = QTextBrowser()
+    text.setOpenExternalLinks(True)
     if type == "text":
         text.setPlainText(txt)
     else:
@@ -63,15 +75,26 @@ def showText(txt, parent=None, type="text", run=True):
     layout.addWidget(text)
     box = QDialogButtonBox(QDialogButtonBox.Close)
     layout.addWidget(box)
-    diag.connect(box, SIGNAL("rejected()"), diag, SLOT("reject()"))
-    diag.setMinimumHeight(400)
-    diag.setMinimumWidth(500)
+    def onReject():
+        if geomKey:
+            saveGeom(diag, geomKey)
+        QDialog.reject(diag)
+    box.rejected.connect(onReject)
+    def onFinish():
+        if geomKey:
+            saveGeom(diag, geomKey)
+    box.accepted.connect(onFinish)
+    diag.setMinimumHeight(minHeight)
+    diag.setMinimumWidth(minWidth)
+    if geomKey:
+        restoreGeom(diag, geomKey)
     if run:
         diag.exec_()
     else:
         return diag, box
 
-def askUser(text, parent=None, help="", defaultno=False, msgfunc=None):
+def askUser(text, parent=None, help="", defaultno=False, msgfunc=None, \
+        title="Anki"):
     "Show a yes/no question. Return true if yes."
     if not parent:
         parent = aqt.mw.app.activeWindow()
@@ -85,8 +108,7 @@ def askUser(text, parent=None, help="", defaultno=False, msgfunc=None):
             default = QMessageBox.No
         else:
             default = QMessageBox.Yes
-        r = msgfunc(parent, "Anki", text, sb,
-                                 default)
+        r = msgfunc(parent, title, text, sb, default)
         if r == QMessageBox.Help:
 
             openHelp(help)
@@ -96,10 +118,10 @@ def askUser(text, parent=None, help="", defaultno=False, msgfunc=None):
 
 class ButtonedDialog(QMessageBox):
 
-    def __init__(self, text, buttons, parent=None, help=""):
+    def __init__(self, text, buttons, parent=None, help="", title="Anki"):
         QDialog.__init__(self, parent)
         self.buttons = []
-        self.setWindowTitle("Anki")
+        self.setWindowTitle(title)
         self.help = help
         self.setIcon(QMessageBox.Warning)
         self.setText(text)
@@ -121,27 +143,29 @@ class ButtonedDialog(QMessageBox):
         if but == "Help":
             # FIXME stop dialog closing?
             openHelp(self.help)
-        return self.clickedButton().text()
+        txt = self.clickedButton().text()
+        # work around KDE 'helpfully' adding accelerators to button text of Qt apps
+        return txt.replace("&", "")
 
     def setDefault(self, idx):
         self.setDefaultButton(self.buttons[idx])
 
-def askUserDialog(text, buttons, parent=None, help=""):
+def askUserDialog(text, buttons, parent=None, help="", title="Anki"):
     if not parent:
         parent = aqt.mw
-    diag = ButtonedDialog(text, buttons, parent, help)
+    diag = ButtonedDialog(text, buttons, parent, help, title=title)
     return diag
 
 class GetTextDialog(QDialog):
 
-    def __init__(self, parent, question, help=None, edit=None, default=u"",
-                 title="Anki"):
+    def __init__(self, parent, question, help=None, edit=None, default="", \
+                 title="Anki", minWidth=400):
         QDialog.__init__(self, parent)
         self.setWindowTitle(title)
         self.question = question
         self.help = help
         self.qlabel = QLabel(question)
-        self.setMinimumWidth(400)
+        self.setMinimumWidth(minWidth)
         v = QVBoxLayout()
         v.addWidget(self.qlabel)
         if not edit:
@@ -157,13 +181,10 @@ class GetTextDialog(QDialog):
         b = QDialogButtonBox(buts)
         v.addWidget(b)
         self.setLayout(v)
-        self.connect(b.button(QDialogButtonBox.Ok),
-                     SIGNAL("clicked()"), self.accept)
-        self.connect(b.button(QDialogButtonBox.Cancel),
-                     SIGNAL("clicked()"), self.reject)
+        b.button(QDialogButtonBox.Ok).clicked.connect(self.accept)
+        b.button(QDialogButtonBox.Cancel).clicked.connect(self.reject)
         if help:
-            self.connect(b.button(QDialogButtonBox.Help),
-                         SIGNAL("clicked()"), self.helpRequested)
+            b.button(QDialogButtonBox.Help).clicked.connect(self.helpRequested)
 
     def accept(self):
         return QDialog.accept(self)
@@ -174,21 +195,26 @@ class GetTextDialog(QDialog):
     def helpRequested(self):
         openHelp(self.help)
 
-def getText(prompt, parent=None, help=None, edit=None, default=u"", title="Anki"):
+def getText(prompt, parent=None, help=None, edit=None, default="",
+            title="Anki", geomKey=None, **kwargs):
     if not parent:
         parent = aqt.mw.app.activeWindow() or aqt.mw
     d = GetTextDialog(parent, prompt, help=help, edit=edit,
-                      default=default, title=title)
+                      default=default, title=title, **kwargs)
     d.setWindowModality(Qt.WindowModal)
+    if geomKey:
+        restoreGeom(d, geomKey)
     ret = d.exec_()
-    return (unicode(d.l.text()), ret)
+    if geomKey and ret:
+        saveGeom(d, geomKey)
+    return (str(d.l.text()), ret)
 
 def getOnlyText(*args, **kwargs):
     (s, r) = getText(*args, **kwargs)
     if r:
         return s
     else:
-        return u""
+        return ""
 
 # fixme: these utilities could be combined into a single base class
 def chooseList(prompt, choices, startrow=0, parent=None):
@@ -205,7 +231,7 @@ def chooseList(prompt, choices, startrow=0, parent=None):
     c.setCurrentRow(startrow)
     l.addWidget(c)
     bb = QDialogButtonBox(QDialogButtonBox.Ok)
-    bb.connect(bb, SIGNAL("accepted()"), d, SLOT("accept()"))
+    bb.accepted.connect(d.accept)
     l.addWidget(bb)
     d.exec_()
     return c.currentRow()
@@ -214,7 +240,8 @@ def getTag(parent, deck, question, tags="user", **kwargs):
     from aqt.tagedit import TagEdit
     te = TagEdit(parent)
     te.setCol(deck)
-    ret = getText(question, parent, edit=te, **kwargs)
+    ret = getText(question, parent, edit=te,
+                  geomKey='getTag', **kwargs)
     te.hideCompleter()
     return ret
 
@@ -230,37 +257,39 @@ def getFile(parent, title, cb, filter="*.*", dir=None, key=None):
     else:
         dirkey = None
     d = QFileDialog(parent)
-    # fix #233 crash
-    if isMac:
-        d.setOptions(QFileDialog.DontUseNativeDialog)
     d.setFileMode(QFileDialog.ExistingFile)
-    d.setDirectory(dir)
+    if os.path.exists(dir):
+        d.setDirectory(dir)
     d.setWindowTitle(title)
     d.setNameFilter(filter)
     ret = []
     def accept():
-        # work around an osx crash
-        #aqt.mw.app.processEvents()
-        file = unicode(list(d.selectedFiles())[0])
+        file = str(list(d.selectedFiles())[0])
         if dirkey:
             dir = os.path.dirname(file)
             aqt.mw.pm.profile[dirkey] = dir
         if cb:
             cb(file)
         ret.append(file)
-    d.connect(d, SIGNAL("accepted()"), accept)
+    d.accepted.connect(accept)
+    if key:
+        restoreState(d, key)
     d.exec_()
+    if key:
+        saveState(d, key)
     return ret and ret[0]
 
 def getSaveFile(parent, title, dir_description, key, ext, fname=None):
     """Ask the user for a file to save. Use DIR_DESCRIPTION as config
     variable. The file dialog will default to open with FNAME."""
     config_key = dir_description + 'Directory'
-    base = aqt.mw.pm.profile.get(config_key, aqt.mw.pm.base)
+
+    defaultPath = QStandardPaths.writableLocation(QStandardPaths.DocumentsLocation)
+    base = aqt.mw.pm.profile.get(config_key, defaultPath)
     path = os.path.join(base, fname)
-    file = unicode(QFileDialog.getSaveFileName(
-        parent, title, path, u"{0} (*{1})".format(key, ext),
-        options=QFileDialog.DontConfirmOverwrite))
+    file = QFileDialog.getSaveFileName(
+        parent, title, path, "{0} (*{1})".format(key, ext),
+        options=QFileDialog.DontConfirmOverwrite)[0]
     if file:
         # add extension
         if not file.lower().endswith(ext):
@@ -280,7 +309,7 @@ def saveGeom(widget, key):
     key += "Geom"
     aqt.mw.pm.profile[key] = widget.saveGeometry()
 
-def restoreGeom(widget, key, offset=None):
+def restoreGeom(widget, key, offset=None, adjustSize=False):
     key += "Geom"
     if aqt.mw.pm.profile.get(key):
         widget.restoreGeometry(aqt.mw.pm.profile[key])
@@ -289,6 +318,9 @@ def restoreGeom(widget, key, offset=None):
                 # bug in osx toolkit
                 s = widget.size()
                 widget.resize(s.width(), s.height()+offset*2)
+    else:
+        if adjustSize:
+            widget.adjustSize()
 
 def saveState(widget, key):
     key += "State"
@@ -320,8 +352,6 @@ def restoreHeader(widget, key):
 def mungeQA(col, txt):
     txt = col.media.escapeImages(txt)
     txt = stripSounds(txt)
-    # osx webkit doesn't understand font weight 600
-    txt = re.sub("font-weight: *600", "font-weight:bold", txt)
     return txt
 
 def applyStyles(widget):
@@ -329,26 +359,12 @@ def applyStyles(widget):
     if os.path.exists(p):
         widget.setStyleSheet(open(p).read())
 
-def getBase(col):
-    base = None
-    mdir = col.media.dir()
-    if isWin and not mdir.startswith("\\\\"):
-        prefix = u"file:///"
-    else:
-        prefix = u"file://"
-    mdir = mdir.replace("\\", "/")
-    base = prefix + unicode(
-        urllib.quote(mdir.encode("utf-8")),
-        "utf-8") + "/"
-    return '<base href="%s">' % base
-
 def openFolder(path):
     if isWin:
-        if isinstance(path, unicode):
-            path = path.encode(sys.getfilesystemencoding())
-        subprocess.Popen(["explorer", path])
+        subprocess.Popen(["explorer", "file://"+path])
     else:
-        QDesktopServices.openUrl(QUrl("file://" + path))
+        with noBundledLibs():
+            QDesktopServices.openUrl(QUrl("file://" + path))
 
 def shortcut(key):
     if isMac:
@@ -365,8 +381,13 @@ def addCloseShortcut(widg):
     if not isMac:
         return
     widg._closeShortcut = QShortcut(QKeySequence("Ctrl+W"), widg)
-    widg.connect(widg._closeShortcut, SIGNAL("activated()"),
-                 widg, SLOT("reject()"))
+    widg._closeShortcut.activated.connect(widg.reject)
+
+def downArrow():
+    if isWin:
+        return "▼"
+    # windows 10 is lacking the smaller arrow on English installs
+    return "▾"
 
 # Tooltips
 ######################################################################
@@ -377,6 +398,7 @@ _tooltipLabel = None
 def tooltip(msg, period=3000, parent=None):
     global _tooltipTimer, _tooltipLabel
     class CustomLabel(QLabel):
+        silentlyClose = True
         def mousePressEvent(self, evt):
             evt.accept()
             self.hide()
@@ -385,7 +407,6 @@ def tooltip(msg, period=3000, parent=None):
     lab = CustomLabel("""\
 <table cellpadding=10>
 <tr>
-<td><img src=":/icons/help-hint.png"></td>
 <td>%s</td>
 </tr>
 </table>""" % msg, aw)
@@ -394,6 +415,7 @@ def tooltip(msg, period=3000, parent=None):
     lab.setWindowFlags(Qt.ToolTip)
     p = QPalette()
     p.setColor(QPalette.Window, QColor("#feffc4"))
+    p.setColor(QPalette.WindowText, QColor("#000000"))
     lab.setPalette(p)
     lab.move(
         aw.mapToGlobal(QPoint(0, -100 + aw.height())))
@@ -423,3 +445,85 @@ def checkInvalidFilename(str, dirsep=True):
                     bad)
         return True
     return False
+
+# Menus
+######################################################################
+
+class MenuList:
+    def __init__(self):
+        self.children = []
+
+    def addItem(self, title, func):
+        item = MenuItem(title, func)
+        self.children.append(item)
+        return item
+
+    def addSeparator(self):
+        self.children.append(None)
+
+    def addMenu(self, title):
+        submenu = SubMenu(title)
+        self.children.append(submenu)
+        return submenu
+
+    def addChild(self, child):
+        self.children.append(child)
+
+    def renderTo(self, qmenu):
+        for child in self.children:
+            if child is None:
+                qmenu.addSeparator()
+            elif isinstance(child, QAction):
+                qmenu.addAction(child)
+            else:
+                child.renderTo(qmenu)
+
+    def popupOver(self, widget):
+        qmenu = QMenu()
+        self.renderTo(qmenu)
+        qmenu.exec_(widget.mapToGlobal(QPoint(0,0)))
+
+    # Chunking
+    ######################################################################
+
+    chunkSize = 30
+
+    def chunked(self):
+        if len(self.children) <= self.chunkSize:
+            return self
+
+        newList = MenuList()
+        oldItems = self.children[:]
+        while oldItems:
+            chunk = oldItems[:self.chunkSize]
+            del oldItems[:self.chunkSize]
+            label = self._chunkLabel(chunk)
+            menu = newList.addMenu(label)
+            menu.children = chunk
+        return newList
+
+    def _chunkLabel(self, items):
+        start = items[0].title
+        end = items[-1].title
+        prefix = os.path.commonprefix([start.upper(), end.upper()])
+        n = len(prefix)+1
+        return "{}-{}".format(start[:n].upper(), end[:n].upper())
+
+class SubMenu(MenuList):
+    def __init__(self, title):
+        super().__init__()
+        self.title = title
+
+    def renderTo(self, menu):
+        submenu = menu.addMenu(self.title)
+        super().renderTo(submenu)
+
+class MenuItem:
+    def __init__(self, title, func):
+        self.title = title
+        self.func = func
+
+    def renderTo(self, qmenu):
+        a = qmenu.addAction(self.title)
+        a.triggered.connect(self.func)
+
