@@ -12,19 +12,23 @@ def assertException(exception, func):
 
 # Creating new decks is expensive. Just do it once, and then spin off
 # copies from the master.
-def getEmptyDeck():
-    if len(getEmptyDeck.master) == 0:
+def getEmptyCol(schedVer=1):
+    if len(getEmptyCol.master) == 0:
         (fd, nam) = tempfile.mkstemp(suffix=".anki2")
         os.close(fd)
         os.unlink(nam)
         col = aopen(nam)
         col.db.close()
-        getEmptyDeck.master = nam
+        getEmptyCol.master = nam
     (fd, nam) = tempfile.mkstemp(suffix=".anki2")
-    shutil.copy(getEmptyDeck.master, nam)
-    return aopen(nam)
+    shutil.copy(getEmptyCol.master, nam)
+    from anki.collection import _Collection
+    _Collection.defaultSchedulerVersion = schedVer
+    col = aopen(nam)
+    _Collection.defaultSchedulerVersion = 1
+    return col
 
-getEmptyDeck.master = ""
+getEmptyCol.master = ""
 
 # Fallback for when the DB needs options passed in.
 def getEmptyDeckWith(**kwargs):
@@ -37,6 +41,6 @@ def getUpgradeDeckPath(name="anki12.anki"):
     src = os.path.join(testDir, "support", name)
     (fd, dst) = tempfile.mkstemp(suffix=".anki2")
     shutil.copy(src, dst)
-    return unicode(dst, "utf8")
+    return dst
 
 testDir = os.path.dirname(__file__)
