@@ -1,20 +1,21 @@
 # coding: utf-8
 
-from tests.shared import getEmptyDeck
+from tests.shared import getEmptyCol
 from anki.utils import stripHTML, joinFields
+import anki.template
 
 def test_modelDelete():
-    deck = getEmptyDeck()
+    deck = getEmptyCol()
     f = deck.newNote()
-    f['Front'] = u'1'
-    f['Back'] = u'2'
+    f['Front'] = '1'
+    f['Back'] = '2'
     deck.addNote(f)
     assert deck.cardCount() == 1
     deck.models.rem(deck.models.current())
     assert deck.cardCount() == 0
 
 def test_modelCopy():
-    deck = getEmptyDeck()
+    deck = getEmptyCol()
     m = deck.models.current()
     m2 = deck.models.copy(m)
     assert m2['name'] == "Basic copy"
@@ -27,10 +28,10 @@ def test_modelCopy():
     assert deck.models.scmhash(m) == deck.models.scmhash(m2)
 
 def test_fields():
-    d = getEmptyDeck()
+    d = getEmptyCol()
     f = d.newNote()
-    f['Front'] = u'1'
-    f['Back'] = u'2'
+    f['Front'] = '1'
+    f['Back'] = '2'
     d.addNote(f)
     m = d.models.current()
     # make sure renaming a field updates the templates
@@ -74,7 +75,7 @@ def test_fields():
     assert d.getNote(d.models.nids(m)[0]).fields == ["", "2", "1"]
 
 def test_templates():
-    d = getEmptyDeck()
+    d = getEmptyCol()
     m = d.models.current(); mm = d.models
     t = mm.newTemplate("Reverse")
     t['qfmt'] = "{{Back}}"
@@ -82,8 +83,8 @@ def test_templates():
     mm.addTemplate(m, t)
     mm.save(m)
     f = d.newNote()
-    f['Front'] = u'1'
-    f['Back'] = u'2'
+    f['Front'] = '1'
+    f['Back'] = '2'
     d.addNote(f)
     assert d.cardCount() == 2
     (c, c2) = f.cards()
@@ -108,20 +109,20 @@ def test_templates():
     assert not d.models.remTemplate(m, m['tmpls'][0])
 
 def test_cloze_ordinals():
-    d = getEmptyDeck()
+    d = getEmptyCol()
     d.models.setCurrent(d.models.byName("Cloze"))
     m = d.models.current(); mm = d.models
     
     #We replace the default Cloze template
     t = mm.newTemplate("ChainedCloze")
-    t['qfmt'] = "{{cloze:text:Text}}"
-    t['afmt'] = "{{text:cloze:Text}}" #independent of the order of mods
+    t['qfmt'] = "{{text:cloze:Text}}"
+    t['afmt'] = "{{text:cloze:Text}}"
     mm.addTemplate(m, t)
     mm.save(m)
     d.models.remTemplate(m, m['tmpls'][0])
     
     f = d.newNote()
-    f['Text'] = u'{{c1::firstQ::firstA}}{{c2::secondQ::secondA}}'
+    f['Text'] = '{{c1::firstQ::firstA}}{{c2::secondQ::secondA}}'
     d.addNote(f)
     assert d.cardCount() == 2
     (c, c2) = f.cards()
@@ -131,22 +132,22 @@ def test_cloze_ordinals():
     
 
 def test_text():
-    d = getEmptyDeck()
+    d = getEmptyCol()
     m = d.models.current()
     m['tmpls'][0]['qfmt'] = "{{text:Front}}"
     d.models.save(m)
     f = d.newNote()
-    f['Front'] = u'hello<b>world'
+    f['Front'] = 'hello<b>world'
     d.addNote(f)
     assert "helloworld" in f.cards()[0].q()
 
 def test_cloze():
-    d = getEmptyDeck()
+    d = getEmptyCol()
     d.models.setCurrent(d.models.byName("Cloze"))
     f = d.newNote()
     assert f.model()['name'] == "Cloze"
     # a cloze model with no clozes is not empty
-    f['Text'] = u'nothing'
+    f['Text'] = 'nothing'
     assert d.addNote(f)
     # try with one cloze
     f = d.newNote()
@@ -187,7 +188,7 @@ def test_cloze():
     assert len(f.cards()) == 2
 
 def test_chained_mods():
-    d = getEmptyDeck()
+    d = getEmptyCol()
     d.models.setCurrent(d.models.byName("Cloze"))
     m = d.models.current(); mm = d.models
     
@@ -210,7 +211,7 @@ def test_chained_mods():
     assert "This <span class=cloze>phrase</span> demonstrates <span class=cloze>en chaine</span> clozes." in f.cards()[0].a()
 
 def test_modelChange():
-    deck = getEmptyDeck()
+    deck = getEmptyCol()
     basic = deck.models.byName("Basic")
     cloze = deck.models.byName("Cloze")
     # enable second template and add a note
@@ -221,8 +222,8 @@ def test_modelChange():
     mm.addTemplate(m, t)
     mm.save(m)
     f = deck.newNote()
-    f['Front'] = u'f'
-    f['Back'] = u'b123'
+    f['Front'] = 'f'
+    f['Back'] = 'b123'
     deck.addNote(f)
     # switch fields
     map = {0: 1, 1: 0}
@@ -267,8 +268,8 @@ def test_modelChange():
     assert f['Back'] == 'f'
     # another note to try model conversion
     f = deck.newNote()
-    f['Front'] = u'f2'
-    f['Back'] = u'b2'
+    f['Front'] = 'f2'
+    f['Back'] = 'b2'
     deck.addNote(f)
     assert deck.models.useCount(basic) == 2
     assert deck.models.useCount(cloze) == 0
@@ -283,8 +284,16 @@ def test_modelChange():
     deck.models.change(cloze, [f.id], basic, map, map)
     assert deck.db.scalar("select count() from cards where nid = ?", f.id) == 1
 
+def test_templates():
+    d = dict(Foo="x", Bar="y")
+    assert anki.template.render("{{Foo}}", d) == "x"
+    assert anki.template.render("{{#Foo}}{{Foo}}{{/Foo}}", d) == "x"
+    assert anki.template.render("{{#Foo}}{{Foo}}{{/Foo}}", d) == "x"
+    assert anki.template.render("{{#Bar}}{{#Foo}}{{Foo}}{{/Foo}}{{/Bar}}", d) == "x"
+    assert anki.template.render("{{#Baz}}{{#Foo}}{{Foo}}{{/Foo}}{{/Baz}}", d) == ""
+
 def test_availOrds():
-    d = getEmptyDeck()
+    d = getEmptyCol()
     m = d.models.current(); mm = d.models
     t = m['tmpls'][0]
     f = d.newNote()
