@@ -1,11 +1,11 @@
 # coding: utf-8
 
 import time
-from tests.shared import  getEmptyDeck
+from tests.shared import  getEmptyCol
 from anki.consts import *
 
 def test_op():
-    d = getEmptyDeck()
+    d = getEmptyCol()
     # should have no undo by default
     assert not d.undoName()
     # let's adjust a study option
@@ -27,7 +27,7 @@ def test_op():
     # and a review will, too
     d.save("add")
     f = d.newNote()
-    f['Front'] = u"one"
+    f['Front'] = "one"
     d.addNote(f)
     d.reset()
     assert d.undoName() == "add"
@@ -36,10 +36,10 @@ def test_op():
     assert d.undoName() == "Review"
 
 def test_review():
-    d = getEmptyDeck()
+    d = getEmptyCol()
     d.conf['counts'] = COUNT_REMAINING
     f = d.newNote()
-    f['Front'] = u"one"
+    f['Front'] = "one"
     d.addNote(f)
     d.reset()
     assert not d.undoName()
@@ -62,7 +62,7 @@ def test_review():
     assert not d.undoName()
     # we should be able to undo multiple answers too
     f = d.newNote()
-    f['Front'] = u"two"
+    f['Front'] = "two"
     d.addNote(f)
     d.reset()
     assert d.sched.counts() == (2, 0, 0)
